@@ -13,10 +13,13 @@
 
 #include "jdlib/misctime.h"
 
+#include "config/globalconf.h"
+
 #include "command.h"
 #include "global.h"
 #include "httpcode.h"
 #include "session.h"
+#include "searchmanager.h"
 
 #include <sstream>
 #include <sys/time.h>
@@ -100,6 +103,7 @@ void ArticleViewMain::reload()
 void ArticleViewMain::show_view()
 {
     m_gotonum_reserve = 0;
+    m_show_instdialog = false;
 
 #ifdef _DEBUG
     std::cout << "ArticleViewBase::show_view\n";
@@ -148,6 +152,11 @@ void ArticleViewMain::show_view()
     // オフラインならダウンロードを開始しない
     if( ! SESSION::is_online() ) return;
 
+    // 板一覧との切り替え方法説明ダイアログ表示
+    if( CONFIG::get_instruct_tglart() && SESSION::get_mode_pane() == SESSION::MODE_2PANE ){
+        m_show_instdialog = true;
+    }
+
     // 差分 download 開始
     get_article()->download_dat();
     if( get_article()->is_loading() ){
@@ -248,6 +257,26 @@ void ArticleViewMain::update_finish()
 
     if( m_gotonum_reserve ) goto_num( m_gotonum_reserve );
     m_gotonum_reserve = 0;
+
+    if( m_show_instdialog ) show_instruct_diag();
+}
+
+
+//
+// 板一覧との切り替え方法説明ダイアログ表示
+//
+void ArticleViewMain::show_instruct_diag()
+{
+    Gtk::MessageDialog mdiag(
+        "スレビューからスレ一覧表示に戻る方法として\n\n(1) マウスジェスチャを使う( 右ボタンを押しながら下にドラッグして離す )\n\n(2) マウスの5ボタンを押す\n\n(3) Alt+x か h か ← を押す\n\n(4) ツールバーのスレ一覧アイコンを押す\n\n(5) 表示メニューからスレ一覧を選ぶ\n\nなどがあります。詳しくはオンラインマニュアルを参照してください。" );
+    Gtk::CheckButton chkbutton( "今後表示しない" );
+    mdiag.get_vbox()->pack_start( chkbutton, Gtk::PACK_SHRINK );
+    mdiag.set_title( "ヒント" );
+    mdiag.show_all_children();
+    mdiag.run();
+
+    if( chkbutton.get_active() ) CONFIG::set_instruct_tglart( false );
+    m_show_instdialog = false;
 }
 
 
@@ -771,3 +800,227 @@ void ArticleViewDrawout::relayout()
 
 
 
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
+// ログキャッシュ検索抽出ビュー
+
+ArticleViewSearchCache::ArticleViewSearchCache( const std::string& url_board,
+                                                const std::string& query, bool mode_or, bool searchall )
+    : ArticleViewBase( url_board ), m_cachetoolbar( NULL )
+    , m_url_board( url_board ), m_mode_or( mode_or ), m_searchall( searchall )
+{
+    struct timeval tv;
+    struct timezone tz;
+    gettimeofday( &tv, &tz );
+
+    //viewのURL更新
+    std::string url_tmp = m_url_board + BOARD_SIGN + KEYWORD_SIGN + "query" + ORMODE_SIGN;
+    if( mode_or ) url_tmp += "1";
+    else url_tmp += "0";
+    url_tmp += TIME_SIGN + MISC::timevaltostr( tv );
+    set_url( url_tmp );
+
+#ifdef _DEBUG
+    std::cout << "ArticleViewSearchCache::ArticleViewSearchCache " << get_url() << std::endl;
+#endif
+
+    setup_view();
+    m_cachetoolbar->m_entry_search.set_text( query );
+
+    CORE::get_search_manager()->sig_search_fin().connect(
+        sigc::mem_fun( *this, &ArticleViewSearchCache::slot_search_fin ) );
+
+    ARTICLE::get_admin()->set_command( "set_tablabel", get_url(), "ログ検索" );
+
+    reload();
+}
+
+
+
+ArticleViewSearchCache::~ArticleViewSearchCache()
+{
+
+#ifdef _DEBUG    
+    std::cout << "ArticleViewSearchCache::~ArticleViewSearchCache : " << get_url() << std::endl;
+#endif
+}
+
+
+//
+// ウィジットのパッキング
+//
+// ArticleViewBase::pack_widget()をオーパロードして検索ツールバーをパック
+//
+void ArticleViewSearchCache::pack_widget()
+{
+    // ツールバーの設定
+    m_cachetoolbar = Gtk::manage( new CacheSearchToolBar() );
+    m_cachetoolbar->m_button_close.signal_clicked().connect( sigc::mem_fun(*this, &ArticleViewSearchCache::close_view ) );
+    m_cachetoolbar->m_button_reload.signal_clicked().connect( sigc::mem_fun(*this, &ArticleViewSearchCache::reload ) );
+    m_cachetoolbar->m_button_stop.signal_clicked().connect( sigc::mem_fun(*this, &ArticleViewSearchCache::stop ) );
+
+    m_cachetoolbar->m_entry_search.signal_activate().connect( sigc::mem_fun( *this, &ArticleViewSearchCache::slot_active_search ) );
+    m_cachetoolbar->m_entry_search.signal_operate().connect( sigc::mem_fun( *this, &ArticleViewSearchCache::slot_entry_operate ) );
+
+    pack_start( *m_cachetoolbar, Gtk::PACK_SHRINK, 2 );
+    pack_start( *drawarea(), Gtk::PACK_EXPAND_WIDGET, 2 );
+
+    show_all_children();
+}
+
+
+//
+// フォーカスイン 
+//
+void ArticleViewSearchCache::focus_view()
+{
+    if( m_url_readcgi.empty() && m_cachetoolbar->m_entry_search.get_text().empty() ) m_cachetoolbar->m_entry_search.grab_focus();
+    else ArticleViewBase::focus_view();
+}
+
+
+//
+// 表示
+//
+void ArticleViewSearchCache::show_view()
+{
+#ifdef _DEBUG
+    std::cout << "ArticleViewSearchCache::show_view()\n";
+#endif
+
+    drawarea()->clear_screen();
+    drawarea()->clear_highlight();
+
+    if( m_searchall ) append_html( "検索対象：キャッシュ内の全ログ<br><br>ある板のログのみを対象に検索する場合はスレ一覧のログ検索ボタンを押してください<br>" );
+    else append_html( "検索対象：" + DBTREE::board_name( m_url_board ) + "<br>" );
+
+    if( CORE::get_search_manager()->get_id() == get_url() ){
+
+        if( CORE::get_search_manager()->is_searching() ){
+            append_html( "検索中・・・" );
+            ARTICLE::get_admin()->set_command( "set_tabicon", get_url(), "loading" );
+        }
+    }
+    else if( CORE::get_search_manager()->is_searching() ){
+        Gtk::MessageDialog mdiag( "他の検索スレッドが実行中です" );
+        mdiag.run();
+    }
+}
+
+
+//
+// 画面を消してレイアウトやりなおし & 再描画
+//
+void ArticleViewSearchCache::relayout()
+{
+#ifdef _DEBUG
+    std::cout << "ArticleViewSearchCache::relayout\n";
+#endif
+
+    std::string query = m_cachetoolbar->m_entry_search.get_text();
+
+    drawarea()->clear_screen();
+    drawarea()->clear_highlight();
+
+    std::ostringstream comment;
+
+    if( m_searchall ) comment <<  "検索対象：キャッシュ内の全ログ<br>";
+    else comment <<  "検索対象：" << DBTREE::board_name( m_url_board ) << "<br>";
+
+    comment << query << " " << m_url_readcgi.size() << " 件<br><br>";
+
+    if( ! m_url_readcgi.empty() ){
+
+        std::list< std::string >::iterator it = m_url_readcgi.begin();
+        for(; it != m_url_readcgi.end(); ++it ){
+
+            DBTREE::ArticleBase* article = DBTREE::get_article( *it );
+            if( article ){
+                comment << "[ " << DBTREE::board_name( article->get_url() ) << " ] ";
+                comment << "<a href=\"" << PROTO_OR << *it + KEYWORD_SIGN + query << "\">"
+                        << article->get_subject() << "</a><br>";
+                comment << *it << "<br><br>";
+            }
+        }
+    }
+
+    append_html( comment.str() );
+    drawarea()->redraw_view();
+}
+
+
+//
+// 検索終了
+//
+void ArticleViewSearchCache::slot_search_fin()
+{
+    if( CORE::get_search_manager()->get_id() != get_url() ) return;
+
+#ifdef _DEBUG
+    std::cout << "ArticleViewSearchCache::slot_search_fin " << get_url() << std::endl;
+#endif
+
+    m_url_readcgi =  CORE::get_search_manager()->get_urllist();
+    relayout();
+
+    ARTICLE::get_admin()->set_command( "set_tabicon", get_url(), "default" );
+}
+
+
+//
+// 再検索
+//
+void ArticleViewSearchCache::reload()
+{
+    if( CORE::get_search_manager()->is_searching() ){
+        Gtk::MessageDialog mdiag( "他の検索スレッドが実行中です" );
+        mdiag.run();
+    }
+
+    std::string query = m_cachetoolbar->m_entry_search.get_text();
+
+    // 検索が終わると ArticleViewSearchCache::slot_search_fin() が呼ばれる
+    if( ! query.empty() ){
+
+        CORE::get_search_manager()->search( get_url(), m_url_board, query, m_mode_or, m_searchall );
+        show_view();
+        focus_view();
+    }
+}
+
+
+//
+// 検索停止
+//
+void ArticleViewSearchCache::stop()
+{
+    CORE::get_search_manager()->stop();
+}
+
+
+
+//
+// 検索バーにフォーカス移動
+//
+void ArticleViewSearchCache::open_searchbar( bool invert )
+{
+    m_cachetoolbar->m_entry_search.grab_focus();     
+}
+
+
+//
+// 検索entryでenterを押した
+//
+void ArticleViewSearchCache::slot_active_search()
+{
+    reload();
+}
+
+//
+// 検索entryの操作
+//
+void ArticleViewSearchCache::slot_entry_operate( int controlid )
+{
+    if( controlid == CONTROL::Cancel ) focus_view();
+    else if( controlid == CONTROL::DrawOutAnd ) reload();
+}
