@@ -1,4 +1,4 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 //
 // メインビュー
@@ -14,13 +14,14 @@ namespace BBSLIST
     // メインビュー
     class BBSListViewMain : public BBSListViewBase
     {
+        bool m_load_etc;
+
       public:
         BBSListViewMain( const std::string& url, const std::string& arg1 = std::string() , const std::string& arg2 = std::string() );
         virtual ~BBSListViewMain();
 
         virtual void shutdown();
 
-        virtual void reload();
         virtual void show_view();
         virtual void update_view();
 
