@@ -1,4 +1,4 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 //#define _DEBUG
 //#define _DEBUG_XML
@@ -75,14 +75,20 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     m_toolbar.m_button_up_search.signal_clicked().connect( sigc::mem_fun( *this, &BBSListViewBase::slot_push_up_search ) );
     m_toolbar.m_button_down_search.signal_clicked().connect( sigc::mem_fun( *this, &BBSListViewBase::slot_push_down_search ) );
     m_toolbar.m_entry_search.signal_operate().connect( sigc::mem_fun( *this, &BBSListViewBase::slot_entry_operate ) );
+    m_toolbar.m_button_close.signal_clicked().connect( sigc::mem_fun( *this, &BBSListViewBase::close_view ) );
+    m_toolbar.m_combo.signal_changed().connect( sigc::mem_fun( *this, &BBSListViewBase::slot_combo_changed ) );
 
     pack_start( m_toolbar, Gtk::PACK_SHRINK );    
     pack_start( m_scrwin );
     show_all_children();    
 
     m_treestore = Gtk::TreeStore::create( m_columns ); 
+
+#if GTKMMVER <= 260
+    // gtkmm26以下にはunset_model()が無いのでここでset_model()しておく
     m_treeview.set_model( m_treestore );
     m_treeview.set_headers_visible( false );
+#endif
 
     // Gtk::TreeStoreでset_fixed_height_mode()を使うとexpandしたときに
     // スクロールバーが誤動作するので使わないこと
@@ -140,7 +146,6 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     action_group()->add( Gtk::Action::create( "CopyURL", "URLをコピー"), sigc::mem_fun( *this, &BBSListViewBase::slot_copy_url ) );
     action_group()->add( Gtk::Action::create( "CopyTitleURL", "タイトルとURLをコピー"), sigc::mem_fun( *this, &BBSListViewBase::slot_copy_title_url ) );
     action_group()->add( Gtk::Action::create( "SelectDir", "ディレクトリ内全選択"), sigc::mem_fun( *this, &BBSListViewBase::slot_select_all_dir ) );
-    action_group()->add( Gtk::Action::create( "Unselect", "選択解除"), sigc::mem_fun( *this, &BBSListViewBase::slot_unselect_all ) );
     action_group()->add( Gtk::Action::create( "PreferenceArticle", "スレのプロパティ"), sigc::mem_fun( *this, &BBSListViewBase::slot_preferences_article ) );
     action_group()->add( Gtk::Action::create( "PreferenceBoard", "板のプロパティ"), sigc::mem_fun( *this, &BBSListViewBase::slot_preferences_board ) );
     action_group()->add( Gtk::Action::create( "PreferenceImage", "画像のプロパティ"), sigc::mem_fun( *this, &BBSListViewBase::slot_preferences_image ) );
@@ -162,9 +167,6 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     "<menuitem action='CopyURL'/>"
     "<menuitem action='CopyTitleURL'/>"
     "<separator/>"
-    "<menuitem action='SelectDir'/>"
-    "<menuitem action='Unselect'/>"
-    "<separator/>"
     "<menuitem action='AppendFavorite'/>"
     "<separator/>"
     "<menuitem action='PreferenceBoard'/>"
@@ -175,12 +177,16 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     "<popup name='popup_menu_mul'>"
     "<menuitem action='OpenRows'/>"
     "<separator/>"
-    "<menuitem action='Unselect'/>"
-    "<separator/>"
     "<menuitem action='AppendFavorite'/>"
     "</popup>"
 
 
+
+    // 通常ディレクトリメニュー
+    "<popup name='popup_menu_dir'>"
+    "<menuitem action='SelectDir'/>"
+    "</popup>"
+
     // お気に入り
     "<popup name='popup_menu_favorite'>"
     "<menuitem action='OpenTab'/>"
@@ -189,9 +195,6 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     "<menuitem action='CopyURL'/>"
     "<menuitem action='CopyTitleURL'/>"
     "<separator/>"
-    "<menuitem action='SelectDir'/>"
-    "<menuitem action='Unselect'/>"
-    "<separator/>"
     "<menuitem action='Rename'/>"
     "<menuitem action='NewDir'/>"
     "<menuitem action='NewCom'/>"
@@ -209,8 +212,6 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     "<popup name='popup_menu_favorite_mul'>"
     "<menuitem action='OpenRows'/>"
     "<separator/>"
-    "<menuitem action='Unselect'/>"
-    "<separator/>"
     "<menu action='Delete_Menu'>"
     "<menuitem action='Delete'/>"
     "</menu>"
@@ -223,18 +224,46 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     "</popup>"
 
 
+    // お気に入りディレクトリメニュー
+    "<popup name='popup_menu_favorite_dir'>"
+    "<menuitem action='SelectDir'/>"
+    "<separator/>"
+    "<menuitem action='Rename'/>"
+    "<menuitem action='NewDir'/>"
+    "<menuitem action='NewCom'/>"
+    "<separator/>"
+    "<menu action='Delete_Menu'>"
+    "<menuitem action='Delete'/>"
+    "</menu>"
+    "</popup>"
+
+    // お気に入りコメントメニュー
+    "<popup name='popup_menu_favorite_com'>"
+    "<menuitem action='Rename'/>"
+    "<menuitem action='NewDir'/>"
+    "<menuitem action='NewCom'/>"
+    "<separator/>"
+    "<menu action='Delete_Menu'>"
+    "<menuitem action='Delete'/>"
+    "</menu>"
+    "</popup>"
+
     // 選択
     "<popup name='popup_menu_select'>"
     "<menuitem action='CopyURL'/>"
     "<menuitem action='CopyTitleURL'/>"
     "<separator/>"
-    "<menuitem action='Unselect'/>"
     "<menuitem action='Rename'/>"
     "<menuitem action='NewDir'/>"
+    "<menuitem action='NewCom'/>"
     "<separator/>"
     "<menu action='Delete_Menu'>"
     "<menuitem action='Delete'/>"
     "</menu>"
+    "<separator/>"
+    "<menuitem action='PreferenceArticle'/>"
+    "<menuitem action='PreferenceBoard'/>"
+    "<menuitem action='PreferenceImage'/>"
     "</popup>"
 
     "</ui>";
@@ -248,6 +277,9 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_mul" ) );
     CONTROL::set_menu_motion( popupmenu );
 
+    popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_dir" ) );
+    CONTROL::set_menu_motion( popupmenu );
+
     popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_favorite" ) );
     CONTROL::set_menu_motion( popupmenu );
 
@@ -257,6 +289,12 @@ BBSListViewBase::BBSListViewBase( const std::string& url,const std::string& arg1
     popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_favorite_space" ) );
     CONTROL::set_menu_motion( popupmenu );
 
+    popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_favorite_dir" ) );
+    CONTROL::set_menu_motion( popupmenu );
+
+    popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_favorite_com" ) );
+    CONTROL::set_menu_motion( popupmenu );
+
     popupmenu = dynamic_cast< Gtk::Menu* >( ui_manager()->get_widget( "/popup_menu_select" ) );
     CONTROL::set_menu_motion( popupmenu );
 
@@ -370,6 +408,9 @@ void BBSListViewBase::focus_view()
     }
 
     m_treeview.grab_focus();
+
+    if( get_url() == URL_BBSLISTVIEW ) get_toolbar().set_combo( COMBO_BBSLIST );
+    if( get_url() == URL_FAVORITEVIEW ) get_toolbar().set_combo( COMBO_FAVORITE );
 }
 
 
@@ -386,6 +427,12 @@ void BBSListViewBase::focus_out()
 
 
 
+void BBSListViewBase::close_view()
+{
+    CORE::core_set_command( "toggle_sidebar" );
+}
+
+
 //
 // viewの操作
 //
@@ -443,7 +490,7 @@ void  BBSListViewBase::operate_view( const int& control )
         case CONTROL::Right:
 
             if( m_treeview.get_row( path ) ){
-                if( ! m_treeview.expand_row( path, false ) ) CORE::core_set_command( "switch_board" );
+                if( ! m_treeview.expand_row( path, false ) ) switch_rightview();
             }
             break;
 
@@ -467,6 +514,10 @@ void  BBSListViewBase::operate_view( const int& control )
             BBSLIST::get_admin()->set_command( "tab_right" );
             break;
 
+        case CONTROL::Quit:
+            close_view();
+            break;
+
         case CONTROL::Reload:
             reload();
             break;
@@ -524,16 +575,16 @@ void  BBSListViewBase::operate_view( const int& control )
 //
 void BBSListViewBase::activate_act_before_popupmenu( const std::string& url )
 {
-    Glib::RefPtr< Gtk::Action > act_selectdir, act_board, act_article, act_image;
-    act_selectdir = action_group()->get_action( "SelectDir" );
+    Glib::RefPtr< Gtk::Action > act_board, act_article, act_image, act_opentab;
     act_board = action_group()->get_action( "PreferenceBoard" );
     act_article = action_group()->get_action( "PreferenceArticle" );
     act_image = action_group()->get_action( "PreferenceImage" );
+    act_opentab = action_group()->get_action( "OpenTab" );
 
-    if( act_selectdir ) act_selectdir->set_sensitive( false );
     if( act_board ) act_board->set_sensitive( false );
     if( act_article ) act_article->set_sensitive( false );
     if( act_image ) act_image->set_sensitive( false );
+    if( act_opentab ) act_opentab->set_sensitive( true );
 
     int type = path2type( m_path_selected );
     switch( type ){
@@ -551,7 +602,11 @@ void BBSListViewBase::activate_act_before_popupmenu( const std::string& url )
             break;
 
         case TYPE_DIR:
-            if( act_selectdir ) act_selectdir->set_sensitive( true );
+            break;
+
+        case TYPE_LINK:
+            if( act_opentab ) act_opentab->set_sensitive( false );
+            break;
     }
 }
 
@@ -625,6 +680,7 @@ void BBSListViewBase::copy_treestore( Glib::RefPtr< Gtk::TreeStore >& store )
 #endif
     m_treestore = store;
     m_treeview.set_model( m_treestore );
+    m_treeview.set_headers_visible( false );
 
     if( m_treestore->children().begin() ){
 
@@ -633,6 +689,7 @@ void BBSListViewBase::copy_treestore( Glib::RefPtr< Gtk::TreeStore >& store )
             m_treeview.collapse_all();
             m_treeview.scroll_to_row( path, 0 );
             m_treeview.get_selection()->unselect_all();
+            m_treeview.set_cursor( path );
         }
     }
 }
@@ -655,7 +712,7 @@ bool BBSListViewBase::slot_button_press( GdkEventButton* event )
     m_dblclick = false;
     if( event->type == GDK_2BUTTON_PRESS ) m_dblclick = true; 
 
-    CORE::core_set_command( "switch_bbslist" );
+    CORE::core_set_command( "switch_sidebar" );
 
     return true;
 }
@@ -702,6 +759,8 @@ bool BBSListViewBase::slot_button_release( GdkEventButton* event )
         SHOW_POPUPMENU( false );
     }
 
+    else operate_view( get_control().button_press( event ) );
+
     event->type = type_copy;
 
     return true;
@@ -777,7 +836,14 @@ bool BBSListViewBase::slot_motion_notify( GdkEventMotion* event )
 //
 bool BBSListViewBase::slot_key_press( GdkEventKey* event )
 {
-    operate_view( SKELETON::View::get_control().key_press( event ) );
+    int key = SKELETON::View::get_control().key_press( event );
+
+    // キー入力でboardを開くとkey_pressイベントがboadviewに送られて
+    // 一番上のスレが開くので、open_row() は slot_key_release() で処理する
+    if( key == CONTROL::OpenBoard ) return true;
+    if( key == CONTROL::OpenBoardTab ) return true;
+
+    operate_view( key );
     return true;
 }
 
@@ -795,6 +861,12 @@ bool BBSListViewBase::slot_key_release( GdkEventKey* event )
     std::cout << "BBSListViewBase::slot_key_release key = " << key << " ctrl = " << ctrl << " shift = " << shift << std::endl;
 #endif
     
+    // キー入力でboardを開くとkey_pressイベントがboadviewに送られて
+    // 一番上のスレが開くので、open_row() は slot_key_release() で処理する
+    int key = SKELETON::View::get_control().key_press( event );
+    if( key == CONTROL::OpenBoard ) operate_view( key );
+    if( key == CONTROL::OpenBoardTab ) operate_view( key );
+
     return true;
 }
 
@@ -967,14 +1039,12 @@ void BBSListViewBase::slot_copy_title_url()
 }
 
 
+
 //
 // ディレクトリ内を全選択
 //
-void BBSListViewBase::slot_select_all_dir()
+void BBSListViewBase::select_all_dir( Gtk::TreeModel::Path path )
 {
-    Gtk::TreeModel::Path path = m_path_selected;
-
-    // とりあえず再帰なしで一階層のみ
     if( is_dir( path ) ){
 
         m_treeview.get_selection()->select( path );
@@ -983,6 +1053,7 @@ void BBSListViewBase::slot_select_all_dir()
         while( m_treeview.get_row( path ) ){
 
             m_treeview.get_selection()->select( path );
+            select_all_dir( path );
             path.next();
         }
     }
@@ -990,15 +1061,16 @@ void BBSListViewBase::slot_select_all_dir()
 
 
 //
-// 選択解除
+// ディレクトリ内を全選択(メニューから呼び出す)
 //
-void BBSListViewBase::slot_unselect_all()
+// m_path_selected にパスをセットしておくこと
+//
+void BBSListViewBase::slot_select_all_dir()
 {
-    m_treeview.get_selection()->unselect_all();
+    select_all_dir( m_path_selected );
 }
 
 
-
 //
 // 板プロパティ表示
 //
@@ -1118,15 +1190,23 @@ void BBSListViewBase::slot_drag_motion( Gtk::TreeModel::Path path )
 {
     if( !m_treeview.get_row( path ) ) return;
     
+    draw_underline( m_drag_path_uline, false );
+
     // 移動先に下線を引く
     int cell_x, cell_y, cell_w, cell_h;
     m_treeview.get_cell_xy_wh( cell_x, cell_y, cell_w, cell_h );
+
+    // 真ん中より上の場合
     if( cell_y < cell_h / 2 ){
+
         Gtk::TreeModel::Path path_tmp = m_treeview.prev_path( path );
         if( m_treeview.get_row( path_tmp ) ) path = path_tmp;
+
+        if( ! is_dir( path ) ) draw_underline( path, true );
+
     }
-    draw_underline( m_drag_path_uline, false );
-    draw_underline( path, true );
+    else draw_underline( path, true );
+
     m_drag_path_uline = path;
 
 #ifdef _DEBUG    
@@ -1277,6 +1357,15 @@ bool BBSListViewBase::open_row( Gtk::TreePath& path, bool tab )
 }
 
 
+//
+// 右のビュー(board)に切り替え
+//
+void BBSListViewBase::switch_rightview()
+{
+    CORE::core_set_command( "switch_rightview" );
+}
+
+
 
 //
 // 選択した行をまとめて開く
@@ -1509,7 +1598,6 @@ bool BBSListViewBase::copy_row( Gtk::TreeModel::iterator& src, Gtk::TreeModel::i
     // destの下にサブディレクトリ作成
     else if( subdir && after && dest_is_dir ){
         it_new = m_treestore->prepend( row_dest.children() );
-        m_treeview.expand_row( GET_PATH( *dest ), false );
     }
 
     // destの後に追加
@@ -1520,7 +1608,6 @@ bool BBSListViewBase::copy_row( Gtk::TreeModel::iterator& src, Gtk::TreeModel::i
 
     Gtk::TreeModel::Row row_tmp = *( it_new );
     setup_row( row_tmp, url, name, type );
-    m_treeview.get_selection()->select( row_tmp );
 
     // srcがdirならサブディレクトリ内の行も再帰的にコピー
     if( src_is_dir ){
@@ -1585,9 +1672,11 @@ void BBSListViewBase::move_selected_row( const Gtk::TreePath& path, bool after )
 
     // 移動開始
 
-    m_treeview.get_selection()->unselect_all();
+    std::list< Gtk::TreeModel::Row > list_destrow;
 
     Gtk::TreeModel::iterator it_dest = m_treestore->get_iter( path );
+    Gtk::TreeModel::iterator it_dest_bkup = it_dest;
+    bool after_bkup = after;
     bool subdir = after;
     it_src = list_it.begin();
     for( int i = 0 ; it_src != list_it.end(); ++i, ++it_src ){
@@ -1597,7 +1686,25 @@ void BBSListViewBase::move_selected_row( const Gtk::TreePath& path, bool after )
         // コピーして削除
         if( copy_row( ( *it_src ), it_dest, subdir, after ) ) m_treestore->erase( ( *it_src ) );
         subdir = false;
-        after = true;;
+        after = true;
+        list_destrow.push_back( *it_dest );
+    }
+
+    // 移動先がディレクトリなら開く
+    if( is_dir( it_dest_bkup ) && after_bkup ) m_treeview.expand_row( GET_PATH( *it_dest_bkup ), false );
+
+    // 範囲選択
+    m_treeview.get_selection()->unselect_all();
+    std::list< Gtk::TreeModel::Row >::iterator it_destrow = list_destrow.begin();
+    for( ; it_destrow != list_destrow.end(); ++it_destrow ){
+
+        Gtk::TreeModel::Row row_tmp = ( *it_destrow );
+        m_treeview.get_selection()->select( row_tmp );
+
+        if( row_tmp[ m_columns.m_type ] == TYPE_DIR ){
+            m_treeview.expand_row( GET_PATH( row_tmp ), false );
+            select_all_dir( GET_PATH( row_tmp ) );
+        }
     }
 }
 
@@ -1706,6 +1813,11 @@ void BBSListViewBase::delete_selected_rows()
         }
     }
 
+    // カーソルを一番最後の列の次に移動する
+    it = list_it.end();
+    Gtk::TreePath next = m_treeview.next_path( GET_PATH( *(--it) ), true );
+    m_treeview.set_cursor( next );
+
     // まとめて削除
     // ディレクトリ内の列を同時に選択している場合があるので後から消す
     it = list_it.end();
@@ -1733,13 +1845,19 @@ std::string BBSListViewBase::tree2xml()
     Gtk::TreePath path = GET_PATH( m_treestore->children().begin() );
     Gtk::TreeModel::Row row;
 
-    // 座標と選択中のパス
-    xml << "<pos";
+    // 座標
+    int y = 0;
     Gtk::Adjustment* adjust = m_treeview.get_vadjustment();
-    if( m_jump_y != -1 ) xml << " y=\"" << m_jump_y << "\"";
-    else if( adjust ) xml << " y=\"" << (int) adjust->get_value() << "\"";
-    else xml << " y=\"0\"";
+    if( adjust ){
 
+        if( m_jump_y != -1 && adjust->get_upper() > m_jump_y ) y = m_jump_y;
+        else  y = ( int ) adjust->get_value();
+    }
+    else if( m_jump_y != -1 ) y = m_jump_y;
+
+    xml << "<pos y=\"" << y << "\"";
+
+    // 選択中のパス
     Gtk::TreeModel::Path focused_path = m_treeview.get_current_path();
     if( !focused_path.empty() ) xml << " path=\"" << focused_path.to_string() << "\"";
     xml << "/>\n";
@@ -1825,7 +1943,12 @@ void BBSListViewBase::xml2tree( const std::string& xml )
     std::list< std::string > lines = MISC::get_lines( xml );
     if( lines.empty() ) return;
 
+#if GTKMMVER >= 280
+    m_treeview.unset_model();
+#endif
+
     // XML を解析してツリーを作る
+    std::list< Gtk::TreePath > list_path_expand;
     Gtk::TreeModel::Row row;
     int y = -1;
     std::string focused_path;
@@ -1876,10 +1999,8 @@ void BBSListViewBase::xml2tree( const std::string& xml )
 
             // expand
             if( regex.str( 1 ) == "1" ){
-
                 Gtk::TreePath path = GET_PATH( row );
-                m_treeview.expand_parents( path );
-                m_treeview.expand_row( path, false );
+                list_path_expand.push_back( path );
             }
 
             // 終わり
@@ -1890,6 +2011,18 @@ void BBSListViewBase::xml2tree( const std::string& xml )
         }
     }
 
+#if GTKMMVER >= 280
+    m_treeview.set_model( m_treestore );
+    m_treeview.set_headers_visible( false );
+#endif
+
+    // ディレクトリオープン
+    std::list< Gtk::TreePath >::iterator it_path = list_path_expand.begin();
+    for( ; it_path != list_path_expand.end(); ++it_path ){
+        m_treeview.expand_parents( *it_path );
+        m_treeview.expand_row( *it_path, false );
+    }
+
     // 前回閉じた位置まで移動
     if( !focused_path.empty() ){
         Gtk::TreePath path = Gtk::TreePath( focused_path );
@@ -2070,3 +2203,20 @@ void BBSListViewBase::slot_entry_operate( int controlid )
 {
     if( controlid == CONTROL::Cancel ) focus_view();
 }
+
+
+
+// ラベルのコンボボックスの表示が変わった
+void BBSListViewBase::slot_combo_changed()
+{
+    switch( get_toolbar().get_combo() ){
+
+        case COMBO_BBSLIST:
+            CORE::core_set_command( "switch_sidebar", URL_BBSLISTVIEW );
+            break;
+
+        case COMBO_FAVORITE:
+            CORE::core_set_command( "switch_sidebar", URL_FAVORITEVIEW );
+            break;
+    }
+}
