@@ -1,4 +1,4 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 //#define _DEBUG
 #include "jddebug.h"
@@ -56,13 +56,42 @@ using namespace BOARD;
 #define DEFAULT_COLMUN_WIDTH 50
 
 enum{
-    COL_MARKVAL_UPDATED = 0,   
-    COL_MARKVAL_CACHED, 
-    COL_MARKVAL_NORMAL,
-    COL_MARKVAL_OLD
+    COL_MARKVAL_OLD = -2,        // dat 落ち
+    COL_MARKVAL_FINISHED = -1,   // キャッシュあり、新着無し、規定スレ数を越えている
+    COL_MARKVAL_NORMAL = 0,      // 通常状態
+    COL_MARKVAL_NEWTHREAD,       // 新スレ
+    COL_MARKVAL_CACHED,          // キャッシュあり、新着無し
+    COL_MARKVAL_UPDATED,         // キャッシュあり、新着有り
+    COL_MARKVAL_BKMARKED,        // ブックマークされている、新着無し
+    COL_MARKVAL_BKMARKED_UPDATED // ブックマークされている、新着有り
 };
 
 
+enum
+{
+    SORTMODE_ASCEND = 0,
+    SORTMODE_DESCEND,
+
+    SORTMODE_MARK1, // 通常
+    SORTMODE_MARK2, // 新着を一番上に
+    SORTMODE_MARK3  // 反転
+};
+
+
+// 昇順で上か下か
+enum{
+    COL_A_UP = -1,
+    COL_B_UP = 1,
+};
+
+
+enum
+{
+    BOOKMARK_AUTO = -1,
+    BOOKMARK_UNSET = 0,
+    BOOKMARK_SET = 1
+};
+
 
 // set_sizing( Gtk::TREE_VIEW_COLUMN_FIXED ) を指定して append_columnする
 #define APPEND_COLUMN(title,model) do{ \
@@ -75,7 +104,11 @@ m_treeview.append_column( *col ); \
 
 BoardView::BoardView( const std::string& url,const std::string& arg1, const std::string& arg2 )
     : SKELETON::View( url ),
-      m_treeview( CONFIG::get_fontname_tree_board(), CONFIG::get_color_back_tree_board() )
+      m_treeview( CONFIG::get_fontname_tree_board(), CONFIG::get_color_back_tree_board() ),
+      m_col( COL_NUM_COL ),
+      m_previous_col( COL_NUM_COL ),
+      m_sortmode( SORTMODE_ASCEND ),
+      m_previous_sortmode( false )
 {
     m_scrwin.add( m_treeview );
     m_scrwin.set_policy( Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS );
@@ -89,6 +122,7 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     m_toolbar.m_button_favorite.signal_clicked().connect( sigc::mem_fun( *this, &BoardView::slot_push_favorite ) );
     m_toolbar.m_button_up_search.signal_clicked().connect( sigc::mem_fun( *this, &BoardView::slot_push_up_search ) );
     m_toolbar.m_button_down_search.signal_clicked().connect( sigc::mem_fun( *this, &BoardView::slot_push_down_search ) );
+    m_toolbar.m_button_search_cache.signal_clicked().connect( sigc::mem_fun( *this, &BoardView::slot_search_cache ) );
     m_toolbar.m_button_preferences.signal_clicked().connect( sigc::mem_fun(*this, &BoardView::slot_push_preferences ) );
     m_toolbar.m_entry_search.signal_operate().connect( sigc::mem_fun( *this, &BoardView::slot_entry_operate ) );
 
@@ -98,10 +132,14 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
 
     // ツリービュー設定
     m_liststore = Gtk::ListStore::create( m_columns );
-    m_treeview.set_model( m_liststore );
 
-#ifndef USE_GTKMM24
-    
+#if GTKMMVER <= 260
+    // gtkmm26以下にはunset_model()が無いのでここでset_model()しておく
+    m_treeview.set_model( m_liststore );
+#endif
+
+#if GTKMMVER >= 260
+
     // セルを固定の高さにする
     // append_column する前に columnに対して set_sizing( Gtk::TREE_VIEW_COLUMN_FIXED ) すること
     m_treeview.set_fixed_height_mode( true );
@@ -125,7 +163,7 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     m_treeview.set_column_for_height( 2 );
 
     // サイズを調整しつつソートの設定
-    for( guint i = 0; i < COL_MARK_VAL; i++ ){
+    for( guint i = 0; i < COL_VISIBLE_END; i++ ){
         
         Gtk::TreeView::Column* column = m_treeview.get_column( i );
 
@@ -166,7 +204,7 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
             break;
 
         case COL_SPEED:
-            width = 1;//SESSION::col_speed(); ぴったり収まるように調整する
+            width = SESSION::col_speed(); 
             break;
         }
 
@@ -176,10 +214,8 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
         column->set_resizable( true );
         column->set_clickable( true );        
 
-        // ヘッダをクリックしたときに呼ぶslotの設定
-        if( i == COL_MARK ) column->signal_clicked().connect( sigc::mem_fun(*this, &BoardView::slot_mark_clicked ) );        
-        else if( i == COL_ID ) column->signal_clicked().connect( sigc::mem_fun(*this, &BoardView::slot_id_clicked ) );
-        else column->signal_clicked().connect( sigc::bind< int >( sigc::mem_fun( *this, &BoardView::slot_col_clicked ), i ) );
+        // ヘッダをクリックしたときに呼ぶslot
+        column->signal_clicked().connect( sigc::bind< int >( sigc::mem_fun( *this, &BoardView::slot_col_clicked ), i ) );
 
         // subjectの背景色設定
         if( i == COL_SUBJECT ){
@@ -192,15 +228,15 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     }
 
     // ソート関数セット
-    m_liststore->set_sort_func( COL_MARK_VAL, sigc::mem_fun( *this, &BoardView::slot_compare_mark_val ) );    
-    m_liststore->set_sort_func( COL_ID, sigc::mem_fun( *this, &BoardView::slot_compare_num_id ) );
-    m_liststore->set_sort_func( COL_SUBJECT, sigc::mem_fun( *this, &BoardView::slot_compare_subject ) );
-    m_liststore->set_sort_func( COL_RES, sigc::mem_fun( *this, &BoardView::slot_compare_num_res ) );
-    m_liststore->set_sort_func( COL_LOAD, sigc::mem_fun( *this, &BoardView::slot_compare_num_load ) );
-    m_liststore->set_sort_func( COL_NEW, sigc::mem_fun( *this, &BoardView::slot_compare_new ) );
-    m_liststore->set_sort_func( COL_SINCE_T, sigc::mem_fun( *this, &BoardView::slot_compare_since_t ) );
-    m_liststore->set_sort_func( COL_WRITE_T, sigc::mem_fun( *this, &BoardView::slot_compare_write_t) );
-    m_liststore->set_sort_func( COL_SPEED, sigc::mem_fun( *this, &BoardView::slot_compare_speed ) );
+    m_liststore->set_sort_func( COL_MARK, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );    
+    m_liststore->set_sort_func( COL_ID, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_SUBJECT, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_RES, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_STR_LOAD, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_STR_NEW, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_SINCE, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_WRITE, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
+    m_liststore->set_sort_func( COL_SPEED, sigc::mem_fun( *this, &BoardView::slot_compare_row ) );
     
     m_treeview.sig_button_press().connect( sigc::mem_fun(*this, &BoardView::slot_button_press ) );
     m_treeview.sig_button_release().connect( sigc::mem_fun(*this, &BoardView::slot_button_release ) );
@@ -218,6 +254,12 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     // ポップアップメニューの設定
     // アクショングループを作ってUIマネージャに登録
     action_group() = Gtk::ActionGroup::create();
+    action_group()->add( Gtk::Action::create( "BookMark", "ブックマーク設定/解除"),
+                         sigc::bind< int >( sigc::mem_fun( *this, &BoardView::slot_bookmark ), BOOKMARK_AUTO ) );
+    action_group()->add( Gtk::Action::create( "SetBookMark", "ブックマーク設定"),
+                         sigc::bind< int >( sigc::mem_fun( *this, &BoardView::slot_bookmark ), BOOKMARK_SET ) );
+    action_group()->add( Gtk::Action::create( "UnsetBookMark", "ブックマーク解除"),
+                         sigc::bind< int >( sigc::mem_fun( *this, &BoardView::slot_bookmark ), BOOKMARK_UNSET ) );
     action_group()->add( Gtk::Action::create( "OpenTab", "タブで開く"), sigc::mem_fun( *this, &BoardView::slot_open_tab ) );
     action_group()->add( Gtk::Action::create( "Favorite_Article", "スレをお気に入りに追加"), sigc::mem_fun( *this, &BoardView::slot_favorite_thread ) );
     action_group()->add( Gtk::Action::create( "Favorite_Board", "板をお気に入りに登録"), sigc::mem_fun( *this, &BoardView::slot_favorite_board ) );
@@ -226,14 +268,13 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     action_group()->add( Gtk::Action::create( "Delete_Menu", "Delete" ) );    
     action_group()->add( Gtk::Action::create( "Delete", "選択した行のログを削除する"), sigc::mem_fun( *this, &BoardView::delete_view ) );
     action_group()->add( Gtk::Action::create( "OpenRows", "選択した行を開く"), sigc::mem_fun( *this, &BoardView::open_selected_rows ) );
-    action_group()->add( Gtk::Action::create( "Unselect", "選択解除"), sigc::mem_fun( *this, &BoardView::slot_unselect_all ) );
     action_group()->add( Gtk::Action::create( "CopyURL", "URLをコピー"), sigc::mem_fun( *this, &BoardView::slot_copy_url ) );
     action_group()->add( Gtk::Action::create( "CopyTitleURL", "タイトルとURLをコピー"), sigc::mem_fun( *this, &BoardView::slot_copy_title_url ) );
     action_group()->add( Gtk::Action::create( "OpenBrowser", "ブラウザで開く"), sigc::mem_fun( *this, &BoardView::slot_open_browser ) );
     action_group()->add( Gtk::Action::create( "AboneThread", "スレをあぼ〜んする"), sigc::mem_fun( *this, &BoardView::slot_abone_thread ) );
     action_group()->add( Gtk::Action::create( "PreferenceArticle", "スレのプロパティ"), sigc::mem_fun( *this, &BoardView::slot_preferences_article ) );
     action_group()->add( Gtk::Action::create( "Preference", "板のプロパティ"), sigc::mem_fun( *this, &BoardView::slot_push_preferences ) );
-
+    action_group()->add( Gtk::Action::create( "SaveDat", "datファイルを保存"), sigc::mem_fun( *this, &BoardView::slot_save_dat ) );
 
     ui_manager() = Gtk::UIManager::create();    
     ui_manager()->insert_action_group( action_group() );
@@ -243,13 +284,15 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
 
     // 通常
     "<popup name='popup_menu'>"
+    "<menuitem action='BookMark'/>"
+    "<separator/>"
     "<menuitem action='OpenTab'/>"
     "<menuitem action='OpenBrowser'/>"
     "<separator/>"
     "<menuitem action='CopyURL'/>"
     "<menuitem action='CopyTitleURL'/>"
     "<separator/>"
-    "<menuitem action='Unselect'/>"
+    "<menuitem action='SaveDat'/>"
     "<separator/>"
     "<menuitem action='Favorite_Article'/>"
     "<separator/>"
@@ -268,7 +311,8 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     "<popup name='popup_menu_mul'>"
     "<menuitem action='OpenRows'/>"
     "<separator/>"
-    "<menuitem action='Unselect'/>"
+    "<menuitem action='SetBookMark'/>"
+    "<menuitem action='UnsetBookMark'/>"
     "<separator/>"
     "<menuitem action='Favorite_Article'/>"
     "<separator/>"
@@ -305,13 +349,13 @@ BoardView::BoardView( const std::string& url,const std::string& arg1, const std:
     CONTROL::set_menu_motion( popupmenu );
 
     // マウスジェスチォ可能
-    SKELETON::View::set_enable_mg( true );
+    set_enable_mg( true );
 
     // オートリロード可
-    SKELETON::View::set_enable_autoreload( true );
+    set_enable_autoreload( true );
 
     // コントロールモード設定
-    SKELETON::View::get_control().set_mode( CONTROL::MODE_BOARD );
+    get_control().set_mode( CONTROL::MODE_BOARD );
 }
 
 
@@ -345,7 +389,7 @@ void BoardView::save_column_width()
     std::cout << "save_column_width " << get_url() << std::endl;
 #endif    
 
-    for( guint i = 0; i < COL_MARK_VAL; i++ ){
+    for( guint i = 0; i < COL_VISIBLE_END; i++ ){
         
         Gtk::TreeView::Column* column = m_treeview.get_column( i );
 
@@ -397,256 +441,204 @@ void BoardView::save_column_width()
 
 
 //
-// ヘッダをクリックしたときのslots
+// ソート実行
 //
-
-// mark をクリック
-void BoardView::slot_mark_clicked()
+void BoardView::exec_sort()
 {
-    // 前回と違う列をソートした後にクリックしたら ASCENDING
-    if( m_previous_col != COL_MARK_VAL ){
-        m_previous_col = COL_MARK_VAL;
-        m_ascend = false;
+    if( m_col < 0 || m_col >= COL_VISIBLE_END ){
+        m_col = COL_ID;
+        m_sortmode = SORTMODE_ASCEND;
     }
 
-    slot_col_clicked( COL_MARK_VAL );
+    m_liststore->set_sort_column( -2, Gtk::SORT_ASCENDING );
+    m_liststore->set_sort_column( m_col, Gtk::SORT_ASCENDING );
+    CORE::core_set_command( "switch_board" );
 }
 
-// id をクリック
-void BoardView::slot_id_clicked()
-{
-    // 前回違う列をソートした後にクリックしたら ASCENDING
-    if( m_previous_col != COL_ID ){
-        m_previous_col = COL_ID;
-        m_ascend = false;
-    }
 
-    slot_col_clicked( COL_ID );
-}
-
-// その他のヘッダをクリック
+//
+// ヘッダをクリックしたときのslot関数
+//
 void BoardView::slot_col_clicked( int col )
 {
-    if( col == COL_STR_LOAD ) col = COL_LOAD;
-    else if( col == COL_STR_NEW ) col = COL_NEW;
-    else if( col == COL_SINCE ) col = COL_SINCE_T;
-    else if( col == COL_WRITE ) col = COL_WRITE_T;
-    
-    DBTREE::board_set_view_sort_column( get_url(), col );
+#ifdef _DEBUG
+    std::cout << "BoardView::slot_col_clicked col = " << col << std::endl;
+#endif
 
-    if( m_previous_col == col && !m_ascend ){
-        m_ascend = true;
-        m_liststore->set_sort_column( col, Gtk::SORT_ASCENDING );
-        DBTREE::board_set_view_sort_ascend( get_url(), true );
+    if( m_col != col ){ // 前回クリックした列と違う列をクリックした
+
+        m_previous_col = m_col;
+        m_previous_sortmode = m_sortmode;
+
+        m_col = col;
+
+        if( m_col == COL_MARK ) m_sortmode = SORTMODE_MARK1;
+        else if( m_col == COL_ID ) m_sortmode = SORTMODE_ASCEND;
+        else if( m_col == COL_SUBJECT ) m_sortmode = SORTMODE_ASCEND;
+        else m_sortmode = SORTMODE_DESCEND;
     }
-    else{
-        m_ascend = false;
-        m_liststore->set_sort_column( col, Gtk::SORT_DESCENDING );
-        DBTREE::board_set_view_sort_ascend( get_url(), false );
+    else if( m_sortmode == SORTMODE_DESCEND ) m_sortmode = SORTMODE_ASCEND;
+    else if( m_sortmode == SORTMODE_ASCEND ) m_sortmode = SORTMODE_DESCEND;
+    else if( m_sortmode == SORTMODE_MARK1 ) m_sortmode = SORTMODE_MARK2;
+    else if( m_sortmode == SORTMODE_MARK2 ) m_sortmode = SORTMODE_MARK3;
+    else if( m_sortmode == SORTMODE_MARK3 ) m_sortmode = SORTMODE_MARK1;
+
+    // 旧バージョンとの互換性のため
+    if( m_col == COL_MARK ){
+        if( m_sortmode == SORTMODE_DESCEND || m_sortmode == SORTMODE_ASCEND ) m_sortmode = SORTMODE_MARK1;
     }
 
-    m_previous_col = col;
+    DBTREE::board_set_view_sort_column( get_url(), m_col );
+    DBTREE::board_set_view_sort_mode( get_url(), m_sortmode );
+    DBTREE::board_set_view_sort_pre_column( get_url(), m_previous_col );
+    DBTREE::board_set_view_sort_pre_mode( get_url(), m_previous_sortmode );
+
+    exec_sort();
 }
 
 
-//
-// mark 列のソート関数
-//
-int BoardView::slot_compare_mark_val( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{
-    Gtk::TreeModel::Row row_a = *( a );
-    Gtk::TreeModel::Row row_b = *( b );
-
-    int num_a = row_a[ m_columns.m_col_mark_val ];
-    int num_b = row_b[ m_columns.m_col_mark_val ];
-
-    // 抽出状態優先
-    int ret = compare_drawbg( row_a, row_b );
-    if( ret ) return ret;
-
-    // 両方ともマーク有り
-    if( num_a < COL_MARKVAL_NORMAL && num_b < COL_MARKVAL_NORMAL ){
-
-        if( num_a > num_b ) return 1;
-        else if( num_a < num_b )return -1;
-
-        // 同じマークならIDで決める
-        return compare_id( row_a, row_b );
-    }
-
-    //　マークが付いている方を優先
-    if( num_a < COL_MARKVAL_NORMAL ) return ( m_ascend ? -1 : 1 );
-    if( num_b < COL_MARKVAL_NORMAL ) return ( m_ascend ? 1 : -1 );
-
-    // どちらもマークが付いていないならIDの小さい方を優先
-    return compare_id( row_a, row_b );
-}
-
-
-// name 列のソート関数
-int BoardView::slot_compare_subject( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{
-    Gtk::TreeModel::Row row_a = *( a ); 
-    Gtk::TreeModel::Row row_b = *( b ); 
-
-    Glib::ustring name_a = row_a[ m_columns.m_col_subject ]; 
-    Glib::ustring name_b = row_b[ m_columns.m_col_subject ]; 
-
-    // 抽出状態優先
-    int ret = compare_drawbg( row_a, row_b );
-    if( ret ) return ret;
-
-    if( name_a > name_b ) return 1;
-    else if( name_a < name_b )return -1;
-
-    // 同じ値ならIDで決める
-    return compare_id( row_a, row_b );
-}
-
-
-#define SLOT_COMPARE_ROW( target ) do{ \
-Gtk::TreeModel::Row row_a = *( a ); \
-Gtk::TreeModel::Row row_b = *( b ); \
-int num_a = row_a[ target ]; \
-int num_b = row_b[ target ]; \
-return compare_row( num_a, num_b, row_a, row_b ); \
-}while(0)
-
-// ID 列のソート関数
-int BoardView::slot_compare_num_id( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_id ); }
-
-// res 列のソート関数
-int BoardView::slot_compare_num_res( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_res ); }
-
-// load 列のソート関数
-int BoardView::slot_compare_num_load( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_load ); }
-
-// new 列のソート関数
-int BoardView::slot_compare_new( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_new ); }
-
-// since 列のソート関数
-int BoardView::slot_compare_since_t( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_since_t ); }
-
-// write 列のソート関数
-int BoardView::slot_compare_write_t( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{ SLOT_COMPARE_ROW( m_columns.m_col_write_t ); }
-
-#undef COMPARE_ROW
-
-// speed 列のソート関数
-int BoardView::slot_compare_speed( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
-{
-    Gtk::TreeModel::Row row_a = *( a ); 
-    Gtk::TreeModel::Row row_b = *( b ); 
-    int num_a = row_a[ m_columns.m_col_speed ]; 
-    int num_b = row_b[ m_columns.m_col_speed ]; 
-
-    // 抽出状態優先
-    int ret = compare_drawbg( row_a, row_b );
-    if( ret ) return ret;
-
-    if( num_a > num_b ) return 1;
-    else if( num_a < num_b )return -1;
-
-    // 同じ値ならIDで決める
-    return compare_id( row_a, row_b );
-}
-
-
-
 //
 // 抽出状態で比較
 //
-// 抽出状態にあるものを上にする。同じなら0
+// row_a が上か　row_b　が上かを返す。同じ状態なら 0
 //
 int BoardView::compare_drawbg( Gtk::TreeModel::Row& row_a, Gtk::TreeModel::Row& row_b )
 {
     bool draw_a = row_a[ m_columns.m_col_drawbg ];
     bool draw_b = row_b[ m_columns.m_col_drawbg ];
-    if( draw_a && !draw_b ) return ( m_ascend ? -1 : 1 );
-    if( draw_b && !draw_a ) return ( m_ascend ? 1 : -1 );
+
+    if( draw_a && ! draw_b ) return COL_A_UP;
+    else if( draw_b && ! draw_a ) return COL_B_UP;
+
     return 0;
 }
 
 
-
 //
-// markで比較
+// 列の値によるソート
 //
-// マークが付いているか、値の小さい方を上にする
+// row_a が上か　row_b　が上かを返す。同じなら 0
 //
-int BoardView::compare_mark( Gtk::TreeModel::Row& row_a, Gtk::TreeModel::Row& row_b )
+int BoardView::compare_col( int col, int sortmode, Gtk::TreeModel::Row& row_a, Gtk::TreeModel::Row& row_b )
 {
-    int num_a = row_a[ m_columns.m_col_mark_val ];
-    int num_b = row_b[ m_columns.m_col_mark_val ];
+    int num_a = 0, num_b = 0;
+    int ret = 0;
 
-    if( num_a < COL_MARKVAL_NORMAL && num_b < COL_MARKVAL_NORMAL ){
+    const int UP = 1;
+    const int DOWN = 2;
 
-        if( num_a > num_b ) return ( m_ascend ? 1 : -1 );
-        else if( num_a < num_b ) return ( m_ascend ? -1 : 1 );
+    switch( col ){
 
-        // 同じマークならIDで決める
-        return compare_id( row_a, row_b );
+        case COL_MARK:
+        {
+            num_a = row_a[ m_columns.m_col_mark_val ];
+            num_b = row_b[ m_columns.m_col_mark_val ];
+
+            if( sortmode == SORTMODE_MARK2 ){ // 新着を一番上に
+
+                if( num_a == COL_MARKVAL_NEWTHREAD
+                    && ( num_b != COL_MARKVAL_NEWTHREAD && num_b != COL_MARKVAL_BKMARKED_UPDATED && num_b != COL_MARKVAL_BKMARKED ) ){
+                    num_a = DOWN; // 下で ret *= -1 しているので UP と DOWNを逆にする
+                    num_b = UP;
+                }
+                else if( num_b == COL_MARKVAL_NEWTHREAD
+                         && ( num_a != COL_MARKVAL_NEWTHREAD && num_a != COL_MARKVAL_BKMARKED_UPDATED && num_a != COL_MARKVAL_BKMARKED ) ){
+                    num_a = UP; // 下で ret *= -1 しているので UP と DOWNを逆にする
+                    num_b = DOWN;
+                }
+            }
+
+            break;
+        }
+
+        case COL_ID:
+            num_a = row_a[ m_columns.m_col_id ];
+            num_b = row_b[ m_columns.m_col_id ];
+            break;
+
+        case COL_SUBJECT:
+        {
+            // 文字列の大小を数字に変換
+            Glib::ustring name_a = row_a[ m_columns.m_col_subject ]; 
+            Glib::ustring name_b = row_b[ m_columns.m_col_subject ]; 
+            if( name_a < name_b ) { num_a = UP; num_b = DOWN; }
+            else if( name_a > name_b ) { num_a = DOWN; num_b = UP; }
+            break;
+        }
+
+        case COL_RES:
+            num_a = row_a[ m_columns.m_col_res ];
+            num_b = row_b[ m_columns.m_col_res ];
+            break;
+
+        case COL_STR_LOAD:
+            num_a = row_a[ m_columns.m_col_load ];
+            num_b = row_b[ m_columns.m_col_load ];
+            break;
+
+        case COL_STR_NEW:
+            num_a = row_a[ m_columns.m_col_new ];
+            num_b = row_b[ m_columns.m_col_new ];
+            break;
+
+        case COL_SINCE:
+            num_a = row_a[ m_columns.m_col_since_t ];
+            num_b = row_b[ m_columns.m_col_since_t ];
+            break;
+
+        case COL_WRITE:
+            num_a = row_a[ m_columns.m_col_write_t ];
+            num_b = row_b[ m_columns.m_col_write_t ];
+            break;
+
+        case COL_SPEED:
+            num_a = row_a[ m_columns.m_col_speed ];
+            num_b = row_b[ m_columns.m_col_speed ];
+            break;
     }
 
-    //　マークが付いている方を優先
-    if( num_a < COL_MARKVAL_NORMAL ) return ( m_ascend ? -1 : 1 );
-    if( num_b < COL_MARKVAL_NORMAL ) return ( m_ascend ? 1 : -1 );
+    // 両方とも 0 より大きいか 0 より小さい場合は普通に比較
+    if( ( num_a > 0 && num_b > 0 ) || ( num_a < 0 && num_b < 0 ) ){
 
-    return 0;
-}
+        if( num_a < num_b ) ret = COL_A_UP;
+        else if( num_a > num_b ) ret = COL_B_UP;
 
-
-//
-// IDで比較
-//
-// IDの小さい方を上にする。両方同じなら 0
-//
-int BoardView::compare_id( Gtk::TreeModel::Row& row_a, Gtk::TreeModel::Row& row_b )
-{
-    int num_a = row_a[ m_columns.m_col_id ];
-    int num_b = row_b[ m_columns.m_col_id ];
-    if( num_a > num_b ) return ( m_ascend ? 1 : -1 );
-    else if( num_a < num_b ) return ( m_ascend ? -1 : 1 );
-    return 0;
-}
-
-
-//
-// ソート関数の本体
-//
-int BoardView::compare_row( int& num_a, int& num_b, Gtk::TreeModel::Row& row_a, Gtk::TreeModel::Row& row_b )
-{
-    // 抽出状態優先
-    int ret = compare_drawbg( row_a, row_b );
-    if( ret ) return ret;
-
-    // 両方とも 0 より大きい
-    if( num_a > 0 && num_b > 0 ){
-
-        // 普通にcompare
-        if( num_a > num_b ) return 1;
-        else if( num_a < num_b )return -1;
-
-        // 同じ値ならIDで決める
-        return compare_id( row_a, row_b );
+        if( sortmode == SORTMODE_DESCEND ) ret *= -1;
+        if( sortmode == SORTMODE_MARK1 || sortmode == SORTMODE_MARK2 ) ret *= -1;
     }
 
     // 0より大きい方を優先
-    if( num_a > 0 ) return ( m_ascend ? -1 : 1 );
-    if( num_b > 0 ) return ( m_ascend ? 1 : -1 );
+    else if( num_a > 0 && num_b <= 0 ) ret = COL_A_UP;
+    else if( num_b > 0 && num_a <= 0 ) ret = COL_B_UP;
 
-    // マークの付いている方を優先
-    ret = compare_mark( row_a, row_b );
-    if( ret ) return ret;
+    // 0を優先
+    else if( num_a == 0 && num_b < 0 ) ret = COL_A_UP;
+    else if( num_b == 0 && num_a < 0 ) ret = COL_B_UP;
 
-    // 両方0以下ならIDで決める
-    return compare_id( row_a, row_b );
+    return ret;
+}
+
+
+//
+// ソート関数
+//
+int BoardView::slot_compare_row( const Gtk::TreeModel::iterator& a, const Gtk::TreeModel::iterator& b )
+{
+    Gtk::TreeModel::Row row_a = *( a );
+    Gtk::TreeModel::Row row_b = *( b );
+
+    // 抽出状態を最優先
+    int ret = compare_drawbg( row_a, row_b ); 
+
+    if( ! ret ) ret = compare_col( m_col, m_sortmode, row_a, row_b );
+
+    // マルチキーソート
+    if( ! ret ) ret = compare_col( m_previous_col, m_previous_sortmode, row_a, row_b );  
+
+    if( ! ret ) ret = compare_col( COL_MARK, SORTMODE_ASCEND, row_a, row_b );
+    if( ! ret ) ret = compare_col( COL_ID, SORTMODE_ASCEND, row_a, row_b );
+
+    return ret;
 }
 
 
@@ -696,14 +688,14 @@ void BoardView::show_view()
 
     // DBに登録されてない
     if( get_url().empty() ){
-        SKELETON::View::set_status( "invalid URL" );
-        BOARD::get_admin()->set_command( "set_status", get_url(), SKELETON::View::get_status() );
+        set_status( "invalid URL" );
+        BOARD::get_admin()->set_command( "set_status", get_url(), get_status() );
         return;
     }
 
     // タイトル表示
-    SKELETON::View::set_title( DBTREE::board_name( get_url() ) );
-    BOARD::get_admin()->set_command( "set_title", get_url(), SKELETON::View::get_title() );
+    set_title( DBTREE::board_name( get_url() ) );
+    BOARD::get_admin()->set_command( "set_title", get_url(), get_title() );
 
     // タブにアイコンを表示
     BOARD::get_admin()->set_command( "set_tabicon", get_url(), "loading" );
@@ -711,14 +703,17 @@ void BoardView::show_view()
     // タブに名前をセット
     BOARD::get_admin()->set_command( "set_tablabel", get_url(), DBTREE::board_name( get_url() ) );
 
+#if GTKMMVER >= 280
+    m_treeview.unset_model();
+#endif
     m_liststore->clear();
     m_pre_query = std::string();
     
     // download 開始
     // 終わったら update_view() が呼ばれる
     DBTREE::board_download_subject( get_url() );
-    SKELETON::View::set_status( "loading..." );
-    BOARD::get_admin()->set_command( "set_status", get_url(), SKELETON::View::get_status() );
+    set_status( "loading..." );
+    BOARD::get_admin()->set_command( "set_status", get_url(), get_status() );
 }
 
 
@@ -735,16 +730,13 @@ void BoardView::redraw_view()
     m_search_invert = false;
 
     // ソート状態回復
-    int col = DBTREE::board_view_sort_column( get_url() );
-    m_previous_col = col;
-    m_ascend = DBTREE::board_view_sort_ascend( get_url() );
-    if( col <= 0 ){
-        m_previous_col = COL_NUM_COL;
-        m_ascend = false;
-        slot_mark_clicked();
-    }
-    else if( m_ascend ) m_liststore->set_sort_column( col, Gtk::SORT_ASCENDING );
-    else m_liststore->set_sort_column( col, Gtk::SORT_DESCENDING );
+    m_col = DBTREE::board_view_sort_column( get_url() );
+    m_sortmode = DBTREE::board_view_sort_mode( get_url() );
+
+    m_previous_col = DBTREE::board_view_sort_pre_column( get_url() );
+    m_previous_sortmode = DBTREE::board_view_sort_pre_mode( get_url() );
+
+    exec_sort();
 
     goto_top();
 }
@@ -777,6 +769,9 @@ void BoardView::update_view()
     bool updated = false;
 
     // 画面消去
+#if GTKMMVER >= 280
+    m_treeview.unset_model();
+#endif
     m_liststore->clear();
 
     // 高速化のためデータベースに直接アクセス
@@ -811,14 +806,17 @@ void BoardView::update_view()
             if( art->get_number_load() && art->get_number() > art->get_number_load() ) updated = true;
         }
 
+#if GTKMMVER >= 280
+        m_treeview.set_model( m_liststore );
+#endif
         redraw_view();
     }
 
     // ステータスバー更新
     std::ostringstream ss_tmp;
     ss_tmp << DBTREE::board_str_code( get_url() ) << " [ 全 " << ( id -1 ) << " ] ";
-    SKELETON::View::set_status( ss_tmp.str() );
-    BOARD::get_admin()->set_command( "set_status", get_url(), SKELETON::View::get_status() );
+    set_status( ss_tmp.str() );
+    BOARD::get_admin()->set_command( "set_status", get_url(), get_status() );
 
     // タブのアイコン状態を更新
     int code = DBTREE::board_code( get_url() );
@@ -921,12 +919,20 @@ void BoardView::operate_view( const int& control )
 
             // Listに戻る
         case CONTROL::Left:
-            CORE::core_set_command( "switch_bbslist" );
+            CORE::core_set_command( "switch_leftview" );
             break;
 
             // 現在の記事を表示
         case CONTROL::Right:
-            CORE::core_set_command( "switch_article" );
+            CORE::core_set_command( "switch_rightview" );
+            break;
+
+        case CONTROL::ScrollLeft:
+            scroll_left();
+            break;
+
+        case CONTROL::ScrollRight:
+            scroll_right();
             break;
 
         case CONTROL::ToggleArticle:
@@ -967,7 +973,7 @@ void BoardView::operate_view( const int& control )
 
         // ポップアップメニュー表示
         case CONTROL::ShowPopupMenu:
-            SKELETON::View::show_popupmenu( "", true );
+            show_popupmenu( "", true );
             break;
 
         // 検索
@@ -1018,6 +1024,57 @@ void BoardView::goto_bottom()
 
 
 
+//
+// 指定したIDのスレに移動
+//
+void BoardView::goto_num( int num )
+{
+    if( ! num ) return;
+
+    focus_view();
+   
+    Gtk::TreeModel::Children child = m_liststore->children();
+    Gtk::TreeModel::Children::iterator it = child.begin();
+    for( ; it != child.end() ; ++it ){
+
+        Gtk::TreeModel::Row row = *( it );
+        
+        if( row[ m_columns.m_col_id ] == num ){
+
+            Gtk::TreePath path = GET_PATH( row );
+            m_treeview.scroll_to_row( path );
+            m_treeview.set_cursor( path );
+            return;
+        }
+    }
+}
+
+
+
+//
+// 左スクロール
+//
+void BoardView::scroll_left()
+{
+    Gtk::Adjustment*  hadjust = m_scrwin.get_hadjustment();
+    if( !hadjust ) return;
+    hadjust->set_value( MAX( 0,  hadjust->get_value() - hadjust->get_step_increment() ) );
+}
+
+
+//
+// 右スクロール
+//
+void BoardView::scroll_right()
+{
+    Gtk::Adjustment*  hadjust = m_scrwin.get_hadjustment();
+    if( !hadjust ) return;
+    hadjust->set_value(  MIN( hadjust->get_upper() - hadjust->get_page_size(),
+                              hadjust->get_value() + hadjust->get_step_increment() ) );
+}
+
+
+
 //
 // 上へ移動
 //
@@ -1158,14 +1215,34 @@ void BoardView::update_row_common( DBTREE::ArticleBase* art, Gtk::TreeModel::Row
     //
     // マーク
 
-    // dat落ち
     int mark_val;
-    if( art->get_status() & STATUS_OLD ){
+
+    // ブックマーク & 新着あり
+    if( art->is_bookmarked_thread() && art->enable_load() ){
+        mark_val = COL_MARKVAL_BKMARKED_UPDATED;
+        row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::BKMARK_UPDATE );
+    }
+
+    // ブックマーク
+    else if( art->is_bookmarked_thread() ){
+        mark_val = COL_MARKVAL_BKMARKED;
+        row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::BKMARK );
+    }
+
+    // dat落ち
+    else if( art->get_status() & STATUS_OLD ){
         mark_val = COL_MARKVAL_OLD;
         row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::DOWN );
     }
+
+    // キャッシュはあるが規定のレス数を越えていて全てのレスが既読
+    else if( art->is_finished() ){
+        mark_val = COL_MARKVAL_FINISHED;
+        row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::CHECK );
+    }
+
     // 新着あり
-    else if( art->enable_load() ){
+    else if( art->is_cached() && art->enable_load() ){
         mark_val = COL_MARKVAL_UPDATED;
         row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::UPDATE );
     }
@@ -1174,6 +1251,11 @@ void BoardView::update_row_common( DBTREE::ArticleBase* art, Gtk::TreeModel::Row
         mark_val = COL_MARKVAL_CACHED;
         row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::CHECK );
     }
+    // キャッシュ無し、新着
+    else if( art->get_hour() < CONFIG::get_newthread_hour() ){
+        mark_val = COL_MARKVAL_NEWTHREAD;
+        row[ m_columns.m_col_mark ] = ICON::get_icon( ICON::NEWTHREAD );
+    }
     //キャッシュ無し
     else{
         mark_val = COL_MARKVAL_NORMAL;
@@ -1201,15 +1283,17 @@ void BoardView::update_row_common( DBTREE::ArticleBase* art, Gtk::TreeModel::Row
 bool BoardView::slot_button_press( GdkEventButton* event )
 {
     // マウスジェスチャ
-    SKELETON::View::get_control().MG_start( event );
+    get_control().MG_start( event );
 
     // ホイールマウスジェスチャ
-    SKELETON::View::get_control().MG_wheel_start( event );
+    get_control().MG_wheel_start( event );
 
     // ダブルクリック
     m_dblclick = false;
     if( event->type == GDK_2BUTTON_PRESS ) m_dblclick = true; 
 
+    CORE::core_set_command( "switch_board" );
+
     return true;
 }
 
@@ -1221,11 +1305,11 @@ bool BoardView::slot_button_press( GdkEventButton* event )
 bool BoardView::slot_button_release( GdkEventButton* event )
 {
     /// マウスジェスチャ
-    int mg = SKELETON::View::get_control().MG_end( event );
+    int mg = get_control().MG_end( event );
 
     // ホイールマウスジェスチャ
     // 実行された場合は何もしない 
-    if( SKELETON::View::get_control().MG_wheel_end( event ) ) return true;
+    if( get_control().MG_wheel_end( event ) ) return true;
 
     if( mg != CONTROL::None && enable_mg() ){
         operate_view( mg );
@@ -1260,8 +1344,8 @@ bool BoardView::slot_button_release( GdkEventButton* event )
         if( m_dblclick ) event->type = GDK_2BUTTON_PRESS;
 
         // スレを開く
-        bool openarticle = SKELETON::View::get_control().button_alloted( event, CONTROL::OpenArticleButton );
-        bool openarticletab = SKELETON::View::get_control().button_alloted( event, CONTROL::OpenArticleTabButton );
+        bool openarticle = get_control().button_alloted( event, CONTROL::OpenArticleButton );
+        bool openarticletab = get_control().button_alloted( event, CONTROL::OpenArticleTabButton );
 
         if( openarticle || openarticletab ){
 
@@ -1272,11 +1356,13 @@ bool BoardView::slot_button_release( GdkEventButton* event )
         }
 
         // ポップアップメニューボタン
-        else if( SKELETON::View::get_control().button_alloted( event, CONTROL::PopupmenuButton ) ){
+        else if( get_control().button_alloted( event, CONTROL::PopupmenuButton ) ){
 
-            SKELETON::View::show_popupmenu( "", false );
+            show_popupmenu( "", false );
         }
 
+        else operate_view( get_control().button_press( event ) );
+
         event->type = type_copy;
     }
 
@@ -1291,7 +1377,7 @@ bool BoardView::slot_button_release( GdkEventButton* event )
 bool BoardView::slot_motion_notify( GdkEventMotion* event )
 {
     /// マウスジェスチャ
-    SKELETON::View::get_control().MG_motion( event );
+    get_control().MG_motion( event );
 
     int x = (int)event->x;
     int y = (int)event->y;
@@ -1321,15 +1407,19 @@ bool BoardView::slot_motion_notify( GdkEventMotion* event )
 //
 bool BoardView::slot_key_press( GdkEventKey* event )
 {
-    int key = SKELETON::View::get_control().key_press( event );
+    int key = get_control().key_press( event );
 
-    // キー入力でスレを開くとkey_releaseイベントがboadviewが画面から
-    // 消えてから送られてWIDGET_REALIZED_FOR_EVENT assertionが出るので
-    // OpenArticle(Tab)は slot_key_release() で処理する
-    if( key == CONTROL::OpenArticle ) return true;
-    if( key == CONTROL::OpenArticleTab ) return true;
+    if( key != CONTROL::None ){
 
-    operate_view( key );
+        // キー入力でスレを開くとkey_releaseイベントがboadviewが画面から
+        // 消えてから送られてWIDGET_REALIZED_FOR_EVENT assertionが出るので
+        // OpenArticle(Tab)は slot_key_release() で処理する
+        if( key == CONTROL::OpenArticle ) return true;
+        if( key == CONTROL::OpenArticleTab ) return true;
+
+        operate_view( key );
+    }
+    else release_keyjump_key( event->keyval );
 
     return true;
 }
@@ -1340,7 +1430,7 @@ bool BoardView::slot_key_press( GdkEventKey* event )
 //
 bool BoardView::slot_key_release( GdkEventKey* event )
 {
-    int key = SKELETON::View::get_control().key_press( event );
+    int key = get_control().key_press( event );
 
     // キー入力でスレを開くとkey_releaseイベントがboadviewが画面から
     // 消えてから送られてWIDGET_REALIZED_FOR_EVENT assertionが出るので
@@ -1359,7 +1449,7 @@ bool BoardView::slot_key_release( GdkEventKey* event )
 bool BoardView::slot_scroll_event( GdkEventScroll* event )
 {
     // ホイールマウスジェスチャ
-    int control = SKELETON::View::get_control().MG_wheel_scroll( event );
+    int control = get_control().MG_wheel_scroll( event );
     if( enable_mg() && control != CONTROL::None ){
         operate_view( control );
         return true;
@@ -1400,6 +1490,28 @@ void BoardView::slot_drag_end()
 
 
 
+//
+// ブックマーク設定、解除
+//
+void BoardView::slot_bookmark( int bookmark )
+{
+    std::string datbase = DBTREE::url_datbase( get_url() );
+    std::list< Gtk::TreeModel::iterator > list_it = m_treeview.get_selected_iterators();
+    std::list< Gtk::TreeModel::iterator >::iterator it = list_it.begin();
+    for( ; it != list_it.end(); ++it ){
+        Gtk::TreeModel::Row row = *( *it );
+        std::string url = datbase + row[ m_columns.m_col_id_dat ];
+        DBTREE::ArticleBase* art = DBTREE::get_article( url );
+        if( art ){
+            bool set = bookmark;
+            if( bookmark == BOOKMARK_AUTO ) set = ! art->is_bookmarked_thread();
+#ifdef _DEBUG
+            std::cout << "BoardView::slot_bookmark url = " << url << " set = " << set << std::endl;
+#endif
+            art->set_bookmarked_thread( set );
+        }
+    }
+}
 
 
 //
@@ -1451,7 +1563,7 @@ void BoardView::slot_new_article()
 //
 void BoardView::slot_push_delete()
 {
-    SKELETON::View::show_popupmenu( "popup_menu_delete", false );
+    show_popupmenu( "popup_menu_delete", false );
 }
 
 
@@ -1460,16 +1572,7 @@ void BoardView::slot_push_delete()
 //
 void BoardView::slot_push_favorite()
 {
-    SKELETON::View::show_popupmenu( "popup_menu_favorite", false );
-}
-
-
-//
-// 選択解除
-//
-void BoardView::slot_unselect_all()
-{
-    m_treeview.get_selection()->unselect_all();
+    show_popupmenu( "popup_menu_favorite", false );
 }
 
 
@@ -1696,16 +1799,24 @@ void BoardView::slot_push_down_search()
 }
 
 
+// キャッシュ検索
+void BoardView::slot_search_cache()
+{
+    std::string query = m_toolbar.m_entry_search.get_text();
+    CORE::core_set_command( "open_article_searchcache", get_url() , query, "false" );
+}
+
+
 //
 // 検索entryの操作
 //
 void BoardView::slot_entry_operate( int controlid )
 {
     if( controlid == CONTROL::Cancel ) focus_view();
+    else if( controlid == CONTROL::SearchCache ) slot_search_cache();
 }
 
 
-
 //
 // 板プロパティ表示
 //
@@ -1731,6 +1842,18 @@ void BoardView::slot_preferences_article()
 }
 
 
+//
+// datを保存
+//
+void BoardView::slot_save_dat()
+{
+    if( m_path_selected.empty() ) return;
+    std::string url = path2daturl( m_path_selected );
+
+    DBTREE::article_save_dat( url, std::string() );
+}
+
+
 //
 // 選択したスレをあぼーん
 //
