@@ -22,6 +22,9 @@ int color_back[ COLOR_SIZE ];
 int color_back_popup[ COLOR_SIZE ];
 int color_back_tree[ COLOR_SIZE ];
 int color_back_tree_board[ COLOR_SIZE ];
+
+bool use_tree_gtkrc;
+
 std::string fontname_main;
 std::string fontname_popup;
 std::string fontname_tree;
@@ -63,6 +66,7 @@ int brownsercombo_id;
 
 int imgpopup_width;
 int imgpopup_height;
+bool use_image_view;
 bool use_mosaic;
 bool zoom_to_fit;
 int del_img_day;
@@ -81,6 +85,10 @@ int margin_popup;
 int mouse_radius;
 int history_size;
 int instruct_popup;
+bool instruct_tglart;
+bool instruct_tglart_end;
+bool instruct_tglimg;
+bool instruct_tglimg_end;
 
 double adjust_underline_pos;
 double adjust_line_space;
@@ -88,6 +96,9 @@ double adjust_line_space;
 bool draw_underline;
 bool strict_char_width;
 
+bool hide_usrcmd;
+int max_show_usrcmd;
+
 int tab_min_str;
 
 bool show_tab_icon;
@@ -108,10 +119,19 @@ bool abone_chain;
 //
 const bool CONFIG::init_config()
 {
-    JDLIB::ConfLoader cf( CACHE::path_conf(), std::string() );
+    std::string path_conf = CACHE::path_conf();
+
+    // 新設定ファイルが無かったら旧ファイルから読み込む
+    if( CACHE::file_exists( path_conf ) != CACHE::EXIST_FILE ){
+
+        if( CACHE::file_exists( CACHE::path_conf_old() ) == CACHE::EXIST_FILE ) path_conf = CACHE::path_conf_old();
+    }
+
+    JDLIB::ConfLoader cf( path_conf, std::string() );
 
 #ifdef _DEBUG
-    std::cout << "CONFIG::init_config empty = " << cf.empty() << std::endl;
+    std::cout << "CONFIG::init_config" << std::endl
+              << "conffile = " << path_conf << " empty = " << cf.empty() << std::endl;
 #endif
 
     // 前回開いたviewを復元するか
@@ -134,8 +154,7 @@ const bool CONFIG::init_config()
     ref_prefix_space = cf.get_option( "ref_prefix_space", 1 );
     for( int i = 0; i < ref_prefix_space; ++i ) ref_prefix += " ";
 
-    // キャッシュのルートディレクトリ
-    // キャッシュ構造は navi2ch の上位互換なので path_cacheroot = "~/.navi2ch/" とすればnavi2chとキャッシュを共有できる
+    // キャッシュのルートディレクトリ(旧バージョンとの互換のため残している)
     path_cacheroot = cf.get_option( "path_cacheroot", "~/.jd/" );
 
     // 読み込み用プロクシとポート番号
@@ -180,6 +199,9 @@ const bool CONFIG::init_config()
     imgpopup_width = cf.get_option( "imgpopup_width", 320 );
     imgpopup_height = cf.get_option( "imgpopup_height", 240 );
 
+    // 画像ビューを使用する
+    use_image_view = cf.get_option( "use_image_view", 1 );
+
     // 画像にモザイクかける
     use_mosaic = cf.get_option( "use_mosaic", 1 );
 
@@ -235,6 +257,9 @@ const bool CONFIG::init_config()
     color_back_popup[ 1 ] = cf.get_option( "color_popup_G", 65000 );
     color_back_popup[ 2 ] = cf.get_option( "color_popup_B", 63000 );
 
+    // ツリービューでgtkrcの設定を使用するか
+    use_tree_gtkrc = cf.get_option( "use_tree_gtkrc", false );
+
     // boardビューで古いスレも表示
     show_oldarticle = cf.get_option( "show_oldarticle", false );
 
@@ -272,6 +297,14 @@ const bool CONFIG::init_config()
     // 0以上なら多重ポップアップの説明を表示する
     instruct_popup = cf.get_option( "instruct_popup", 100 );    
 
+    // スレビューを開いたときにスレ一覧との切り替え方法を説明する
+    instruct_tglart = cf.get_option( "instruct_tglart", true );
+    instruct_tglart_end = false;
+
+    // 画像ビューを開いたときにスレビューとの切り替え方法を説明する
+    instruct_tglimg = cf.get_option( "instruct_tglimg", true );
+    instruct_tglimg_end = false;
+    
     // スレ表示の行間調整
     adjust_underline_pos = cf.get_option( "adjust_underline_pos", 1.0 );
     adjust_line_space = cf.get_option( "adjust_line_space", 1.0 );
@@ -282,6 +315,12 @@ const bool CONFIG::init_config()
     // スレビューで文字幅の近似を厳密にする
     strict_char_width = cf.get_option( "strict_char_width", false );
 
+    // ユーザーコマンドで選択できない項目を非表示にする
+    hide_usrcmd = cf.get_option( "hide_usrcmd", false );
+
+    // 指定した数よりもユーザーコマンドが多い場合はサブメニュー化する
+    max_show_usrcmd = cf.get_option( "max_show_usrcmd", 3 );
+
     // タブに表示する文字列の最小値
     tab_min_str = cf.get_option( "tab_min_str", 4 );
 
@@ -325,7 +364,17 @@ const bool CONFIG::init_config()
 // 
 void CONFIG::save_conf()
 {
-    JDLIB::ConfLoader cf( CACHE::path_conf(), std::string() );
+    save_conf_impl( CACHE::path_conf() );
+    if( CACHE::file_exists( CACHE::path_conf_old() ) == CACHE::EXIST_FILE ) save_conf_impl( CACHE::path_conf_old() );
+}
+
+void CONFIG::save_conf_impl( const std::string& path )
+{
+#ifdef _DEBUG
+    std::cout << "CONFIG::save_conf_impl path = " << path << std::endl;
+#endif
+
+    JDLIB::ConfLoader cf( path, std::string() );
 
     cf.update( "restore_board", restore_board );
     cf.update( "restore_article", restore_article );
@@ -372,6 +421,7 @@ void CONFIG::save_conf()
 
     cf.update( "imgpopup_width", imgpopup_width );
     cf.update( "imgpopup_height", imgpopup_height );
+    cf.update( "use_image_view", use_image_view );
     cf.update( "use_mosaic", use_mosaic );
     cf.update( "zoom_to_fit", zoom_to_fit );
     cf.update( "del_img_day", del_img_day );
@@ -405,6 +455,8 @@ void CONFIG::save_conf()
     cf.update( "color_popup_G", color_back_popup[ 1 ] );
     cf.update( "color_popup_B", color_back_popup[ 2 ] );
 
+    cf.update( "use_tree_gtkrc", use_tree_gtkrc );
+
     cf.update( "show_oldarticle", show_oldarticle );
     cf.update( "newthread_hour", newthread_hour );
 
@@ -417,6 +469,8 @@ void CONFIG::save_conf()
     cf.update( "mouse_radius", mouse_radius );
     cf.update( "history_size", history_size );
     cf.update( "instruct_popup", instruct_popup );
+    cf.update( "instruct_tglart", instruct_tglart );
+    cf.update( "instruct_tglimg", instruct_tglimg );
 
     cf.update( "adjust_underline_pos", adjust_underline_pos );
     cf.update( "adjust_line_space", adjust_line_space );
@@ -424,6 +478,9 @@ void CONFIG::save_conf()
     cf.update( "draw_underline", draw_underline );
     cf.update( "strict_char_width", strict_char_width );
 
+    cf.update( "hide_usrcmd", hide_usrcmd );
+    cf.update( "max_show_usrcmd", max_show_usrcmd );
+
     cf.update( "tab_min_str", tab_min_str );
 
     cf.update( "show_tab_icon", show_tab_icon );
@@ -474,6 +531,7 @@ void CONFIG::set_color_back_popup( int* color ) { memcpy( color_back_popup, colo
 void CONFIG::set_color_back_tree( int* color ) { memcpy( color_back_tree, color, sizeof( int )*COLOR_SIZE ); }
 void CONFIG::set_color_back_tree_board( int* color ) { memcpy( color_back_tree_board, color, sizeof( int )*COLOR_SIZE ); }
 
+const bool CONFIG::get_use_tree_gtkrc(){ return use_tree_gtkrc; }
 
 const std::string& CONFIG::get_fontname_main() { return fontname_main; }
 const std::string& CONFIG::get_fontname_popup() { return fontname_popup; }
@@ -491,7 +549,6 @@ const std::string&  CONFIG::get_ref_prefix(){ return ref_prefix; }
 
 const std::string& CONFIG::get_url_login2ch() { return url_login2ch; }
 const std::string& CONFIG::get_url_bbsmenu() { return url_bbsmenu; }
-const std::string& CONFIG::get_path_cacheroot() { return path_cacheroot; }
 
 const std::string& CONFIG::get_agent_for2ch() { return agent_for2ch; }
 
@@ -536,6 +593,8 @@ void CONFIG::set_brownsercombo_id( int id ){ brownsercombo_id = id; }
 
 const int CONFIG::get_imgpopup_width(){ return imgpopup_width; }
 const int CONFIG::get_imgpopup_height(){ return imgpopup_height; }
+const bool CONFIG::get_use_image_view(){ return use_image_view; }
+void CONFIG::set_use_image_view( bool image_view ){ use_image_view = image_view; }
 const bool CONFIG::get_use_mosaic(){ return use_mosaic; }
 void CONFIG::set_use_mosaic( bool mosaic ) { use_mosaic = mosaic; }
 const bool CONFIG::get_zoom_to_fit(){ return zoom_to_fit; }
@@ -568,11 +627,38 @@ const int CONFIG::get_instruct_popup(){
     return 0;
 }
 
+
+const bool CONFIG::get_instruct_tglart(){
+
+    if( instruct_tglart_end ) return false;
+
+    instruct_tglart_end = true; // 一度表示したら表示しない
+    return instruct_tglart;
+}
+void CONFIG::set_instruct_tglart( bool tgl ){ instruct_tglart = tgl; }
+
+const bool CONFIG::get_instruct_tglimg(){
+
+    if( instruct_tglimg_end ) return false;
+
+    instruct_tglimg_end = true; // 一度表示したら表示しない
+    return instruct_tglimg;
+}
+
+void CONFIG::set_instruct_tglimg( bool tgl ){ instruct_tglimg = tgl; }
+
+
 const double CONFIG::get_adjust_underline_pos(){ return adjust_underline_pos; }
 const double CONFIG::get_adjust_line_space(){ return adjust_line_space; }
 
 const bool CONFIG::get_draw_underline(){ return draw_underline; }
+
 const bool CONFIG::get_strict_char_width(){ return strict_char_width; }
+void CONFIG::set_strict_char_width( bool strictwidth ){ strict_char_width = strictwidth; }
+
+const bool CONFIG::get_hide_usrcmd(){ return hide_usrcmd; }
+const int CONFIG::get_max_show_usrcmd(){ return max_show_usrcmd; }
+
 
 const int CONFIG::get_tab_min_str(){ return tab_min_str; }
 
