@@ -15,6 +15,7 @@ namespace CONFIG
     const bool init_config();
 
     void save_conf();
+    void save_conf_impl( const std::string& path );
     
     // 前回開いたviewを復元するか
     const bool get_restore_board();
@@ -41,6 +42,9 @@ namespace CONFIG
     void set_color_back_tree( int* color );
     void set_color_back_tree_board( int* color );
 
+    // ツリービューでgtkrcの設定を使用するか
+    const bool get_use_tree_gtkrc();
+
     // フォント
     const std::string& get_fontname_main();
     const std::string& get_fontname_popup();
@@ -63,10 +67,6 @@ namespace CONFIG
     // bbsmenu.htmlのURL
     const std::string& get_url_bbsmenu();    
 
-    // キャッシュのルートディレクトリ
-    // キャッシュ構造は navi2ch の上位互換なので path_cacheroot = "~/.navi2ch/" とすればnavi2chとキャッシュを共有できる
-    const std::string& get_path_cacheroot();
-
     // 2ch にアクセスするときのエージェント名
     const std::string& get_agent_for2ch();
 
@@ -123,6 +123,10 @@ namespace CONFIG
     const int get_imgpopup_width();
     const int get_imgpopup_height();
 
+    // 画像ビューを使用する
+    const bool get_use_image_view();
+    void set_use_image_view( bool image_view );
+
     // 画像にモザイクかける
     const bool get_use_mosaic();
     void set_use_mosaic( bool mosaic );
@@ -174,6 +178,14 @@ namespace CONFIG
     // 0以上なら多重ポップアップの説明を表示する
     const int get_instruct_popup();
 
+    // スレビューを開いたときにスレ一覧との切り替え方法を説明する
+    const bool get_instruct_tglart();
+    void set_instruct_tglart( bool set );
+
+    // 画像ビューを開いたときにスレビューとの切り替え方法を説明する
+    const bool get_instruct_tglimg();
+    void set_instruct_tglimg( bool set );
+
     // スレ表示の行間調整
     const double get_adjust_underline_pos();
     const double get_adjust_line_space();
@@ -183,6 +195,13 @@ namespace CONFIG
 
     // スレビューで文字幅の近似を厳密にする
     const bool get_strict_char_width();
+    void set_strict_char_width( bool strictwidth );
+
+    // ユーザーコマンドで選択できない項目を非表示にする
+    const bool get_hide_usrcmd();
+
+    // 指定した数よりもユーザーコマンドが多い場合はサブメニュー化する
+    const int get_max_show_usrcmd();
 
     // タブに表示する文字列の最小値
     const int get_tab_min_str();
