@@ -283,18 +283,41 @@ void DBTREE::board_set_view_sort_column( const std::string& url, int column )
 }
 
 
-const bool DBTREE::board_view_sort_ascend( const std::string& url )
+const int DBTREE::board_view_sort_mode( const std::string& url )
 {
-    return DBTREE::get_board( url )->get_view_sort_ascend();
+    return DBTREE::get_board( url )->get_view_sort_mode();
 }
 
 
-void DBTREE::board_set_view_sort_ascend( const std::string& url, bool ascend )
+void DBTREE::board_set_view_sort_mode( const std::string& url, int mode )
 {
-    DBTREE::get_board( url )->set_view_sort_ascend( ascend );
+    DBTREE::get_board( url )->set_view_sort_mode( mode );
 }
 
 
+const int DBTREE::board_view_sort_pre_column( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_view_sort_pre_column();
+}
+
+
+void DBTREE::board_set_view_sort_pre_column( const std::string& url, int column )
+{
+    DBTREE::get_board( url )->set_view_sort_pre_column( column );
+}
+
+
+const int DBTREE::board_view_sort_pre_mode( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_view_sort_pre_mode();
+}
+
+
+void DBTREE::board_set_view_sort_pre_mode( const std::string& url, int mode )
+{
+    DBTREE::get_board( url )->set_view_sort_pre_mode( mode );
+}
+
 const bool DBTREE::board_check_noname( const std::string& url )
 {
     return DBTREE::get_board( url )->get_check_noname();
@@ -306,6 +329,102 @@ void DBTREE::board_set_check_noname( const std::string& url, bool check )
     DBTREE::get_board( url )->set_check_noname( check );
 }
 
+const int DBTREE::board_get_mode_local_proxy( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_mode_local_proxy();
+}
+
+const std::string& DBTREE::board_get_local_proxy( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_local_proxy();
+}
+
+const int DBTREE::board_get_local_proxy_port( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_local_proxy_port();
+}
+
+void DBTREE::board_set_mode_local_proxy( const std::string& url, int mode )
+{
+    DBTREE::get_board( url )->set_mode_local_proxy( mode );
+}
+
+void DBTREE::board_set_local_proxy( const std::string& url, const std::string& proxy )
+{
+    DBTREE::get_board( url )->set_local_proxy( proxy );
+}
+
+void DBTREE::board_set_local_proxy_port( const std::string& url, int port )
+{
+    DBTREE::get_board( url )->set_local_proxy_port( port );
+}
+
+const int DBTREE::board_get_mode_local_proxy_w( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_mode_local_proxy_w();
+}
+
+const std::string& DBTREE::board_get_local_proxy_w( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_local_proxy_w();
+}
+
+const int DBTREE::board_get_local_proxy_port_w( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_local_proxy_port_w();
+}
+
+void DBTREE::board_set_mode_local_proxy_w( const std::string& url, int mode )
+{
+    DBTREE::get_board( url )->set_mode_local_proxy_w( mode );
+}
+
+void DBTREE::board_set_local_proxy_w( const std::string& url, const std::string& proxy )
+{
+    DBTREE::get_board( url )->set_local_proxy_w( proxy );
+}
+
+void DBTREE::board_set_local_proxy_port_w( const std::string& url, int port )
+{
+    DBTREE::get_board( url )->set_local_proxy_port_w( port );
+}
+
+const std::string& DBTREE::board_get_write_name( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_write_name();
+}
+
+const std::string& DBTREE::board_get_write_mail( const std::string& url )
+{
+    return DBTREE::get_board( url )->get_write_mail();
+}
+
+void DBTREE::board_set_write_name( const std::string& url, const std::string& name )
+{
+    DBTREE::get_board( url )->set_write_name( name );
+}
+
+void DBTREE::board_set_write_mail( const std::string& url, const std::string& mail )
+{
+    DBTREE::get_board( url )->set_write_mail( mail );
+}
+
+
+void DBTREE::read_boardinfo_all()
+{
+    DBTREE::get_root()->read_boardinfo_all();
+}
+
+std::list< std::string > DBTREE::search_cache_all( const std::string& url, const std::string& query, bool mode_or, bool& stop )
+{
+    return DBTREE::get_root()->search_cache( query, mode_or, stop );
+}
+
+std::list< std::string > DBTREE::search_cache( const std::string& url, const std::string& query, bool mode_or, bool& stop )
+{
+    return DBTREE::get_board( url )->search_cache( query, mode_or, stop );
+}
+
 
 void DBTREE::update_abone_all_board()
 {
@@ -635,11 +754,25 @@ std::list< std::string > DBTREE::get_abone_list_regex( const std::string& url )
     return DBTREE::get_article( url )->get_abone_list_regex();
 }
 
+
+std::vector< char > DBTREE::get_abone_vec_res( const std::string& url )
+{
+    return DBTREE::get_article( url )->get_abone_vec_res();
+}
+
+
 void DBTREE::reset_abone( const std::string& url, std::list< std::string >& ids, std::list< std::string >& names
                           , std::list< std::string >& words, std::list< std::string >& regexs
+                          , std::vector< char >& vec_abone_res
                           , bool transparent, bool chain)
 {
-    DBTREE::get_article( url )->reset_abone( ids, names, words, regexs, transparent, chain );
+    DBTREE::get_article( url )->reset_abone( ids, names, words, regexs, vec_abone_res, transparent, chain );
+}
+
+
+void DBTREE::set_abone_res( const std::string& url, const int number, const bool set )
+{
+    DBTREE::get_article( url )->set_abone_res( number, set );
 }
 
 
