@@ -1,4 +1,4 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 //#define _DEBUG
 //#define _SHOW_BOARD
@@ -15,6 +15,8 @@
 #include "jdlib/miscmsg.h"
 #include "jdlib/loaderdata.h"
 
+#include "skeleton/editviewdialog.h"
+
 #include "global.h"
 #include "command.h"
 #include "config/globalconf.h"
@@ -99,6 +101,8 @@ BoardBase* Root::get_board( const std::string& url, int count )
     std::cout << "Root::get_board : count = " << count << " url = " << url << std::endl;
 #endif
 
+    const int max_count = 50;
+
     // キャッシュ
     if( url == m_get_board_url && m_get_board ){
 #ifdef _DEBUG
@@ -109,13 +113,8 @@ BoardBase* Root::get_board( const std::string& url, int count )
     m_get_board_url = url;
     m_get_board = NULL;
 
-    if( count > 20 ){
-        MISC::ERRMSG( "Root::get_board: could not find board" );
-        return m_board_null;
-    }
-
     // 先頭が http:// でなかったら足して再帰呼び出し
-    if( url.find( "http://" ) != 0 ) return get_board( "http://" + url , count + 1 );
+    if( count < max_count && url.find( "http://" ) != 0 ) return get_board( "http://" + url , count + 1 );
 
     // サーチ
     std::list< BoardBase* >::iterator it;
@@ -131,9 +130,10 @@ BoardBase* Root::get_board( const std::string& url, int count )
 
 
     // 移転した時はrootを付け変えて再帰呼び出し
-    std::string new_url = is_board_moved( url );
-    if( ! new_url.empty() ) return get_board( new_url, count + 1 );
-
+    if( count < max_count ){
+        std::string new_url = is_board_moved( url );
+        if( ! new_url.empty() ) return get_board( new_url, count + 1 );
+    }
 
     // 2ch型の場合、板パスを見てもし一致したら新ホストに移転したと判断して移転テーブルを更新する
     if( is_2ch( url ) ){
@@ -143,20 +143,41 @@ BoardBase* Root::get_board( const std::string& url, int count )
 
             BoardBase* board = *( it );
 
-            if( url.find( board->get_path_board() + "/" ) != std::string::npos ){
+            if( is_2ch( board->get_root() ) && url.find( board->get_path_board() + "/" ) != std::string::npos ){
 
                 // 板移転テーブルを更新
-                MOVETABLE movetable;
-                movetable.old_root = MISC::get_hostname( url );
-                movetable.new_root = board->get_root();
-                movetable.path_board = board->get_path_board();
-                m_movetable.push_back( movetable );
 
-                std::ostringstream ss;
-                ss << board->get_name() << " is moved" << std::endl
-                   << "old url = " << movetable.old_root + board->get_path_board() << "/" << std::endl
-                   << "new url = " << board->url_boardbase() << std::endl;
-                MISC::MSG( ss.str() );
+                if( count < max_count ){ 
+
+                    MOVETABLE movetable;
+                    movetable.old_root = MISC::get_hostname( url );
+                    movetable.new_root = board->get_root();
+                    movetable.path_board = board->get_path_board();
+                    m_movetable.push_back( movetable );
+
+                    std::ostringstream ss;
+                    ss << board->get_name() << std::endl
+                       << "旧 URL = " << movetable.old_root + board->get_path_board() << "/" << std::endl
+                       << "新 URL  = " << board->url_boardbase() << std::endl;
+                    MISC::MSG( ss.str() );
+
+                }
+                else{ // 移転テーブルが循環している場合
+
+                    std::string str = "移転テーブルが破損していたので修復しました\n";
+
+                    std::list< MOVETABLE >::iterator it_move = m_movetable.begin();
+                    for( ; it_move != m_movetable.end(); ++it_move ){
+
+                        if( is_2ch( ( *it_move ).old_root ) && url.find( ( *it_move ).path_board + "/" ) != std::string::npos ){
+
+                            ( *it_move ).new_root = board->get_root();
+                            str += ( *it_move ).old_root + board->get_path_board() + "/ -> " + board->url_boardbase() + "\n";
+                        }
+                    }
+
+                    MISC::MSG( str );
+                }
 
                 //移転テーブル保存
                 save_movetable();
@@ -168,7 +189,7 @@ BoardBase* Root::get_board( const std::string& url, int count )
         }
 
         // 最後が "/" で終わってなかったら足して再帰呼び出し
-        if( url[ url.length() ] != '/' ) return get_board( url + "/" , count + 1 );
+        if( count < max_count && url[ url.length() -1 ] != '/' ) return get_board( url + "/" , count + 1 );
     }
 
     // それでも見つからなかったらNULLクラスを返す
@@ -365,7 +386,9 @@ void Root::update_boards( const std::string xml )
     // 移転があった
     if( ! m_move_info.empty() ){
         
-        Gtk::MessageDialog mdiag( "移転一覧\n" + m_move_info  ); mdiag.run();
+        SKELETON::EditViewDialog diag( m_move_info, "移転板一覧", false );
+        diag.resize( 600, 400 );
+        diag.run();
 
         //移転テーブル保存
         save_movetable();
@@ -382,7 +405,7 @@ void Root::update_boards( const std::string xml )
 bool Root::set_board( const std::string& url, const std::string& name, const std::string& basicauth, bool etc )
 {
 #ifdef _SHOW_BOARD
-    std::cout << url << " " << name << std::endl;
+    std::cout << "Root::set_board " << url << " " << name << std::endl;
 #endif
 
     JDLIB::Regex regex;
@@ -438,7 +461,7 @@ bool Root::set_board( const std::string& url, const std::string& name, const std
 
 #ifdef _SHOW_BOARD
     std::cout << "root = " << root << " path_board = " << path_board
-              << "basicauth = " << basicauth <<" type = " << type << " stat = " << stat << std::endl;
+              << " basicauth = " << basicauth <<" type = " << type << " stat = " << stat << std::endl;
 #endif
 
     // 新板登録
@@ -462,19 +485,19 @@ bool Root::set_board( const std::string& url, const std::string& name, const std
         std::string new_path = CACHE::path_board_root( new_url );
 
         std::ostringstream ss;
-        ss << board->get_name() << " is moved" << std::endl
-           << " old url = " << old_url << std::endl
-           << " new url = " << new_url << std::endl;
+        ss << board->get_name() << std::endl
+           << " 旧 URL = " << old_url << std::endl
+           << " 新 URL = " << new_url << std::endl;
         MISC::MSG( ss.str() );
 
         // もしキャッシュが存在したら移動して移転テーブル更新
-        if( CACHE::is_file_exists( old_path ) == CACHE::EXIST_DIR ){
+        if( CACHE::file_exists( old_path ) == CACHE::EXIST_DIR ){
 
             // キャッシュがある場合はダイアログに表示
-            m_move_info += ss.str();
+            m_move_info += ss.str() + "\n";
 
             // 移動先に同名のファイルかフォルダ何かあったらリネームしてバックアップをとっておく
-            if( CACHE::is_file_exists( new_path ) != CACHE::EXIST_ERROR ){
+            if( CACHE::file_exists( new_path ) != CACHE::EXIST_ERROR ){
 
                 std::string path_tmp = new_path.substr( 0, new_path.length() - 1 ) + "_bk/";
                 if( rename( new_path.c_str(), path_tmp.c_str() ) == 0 ) MISC::MSG( "rename : " +  new_path + " -> " + path_tmp );
@@ -551,7 +574,7 @@ int Root::is_moved( const std::string& root,
 //
 void Root::load_etc()
 {
-    m_xml_etc.clear();
+    m_xml_etc = std::string();
 
     JDLIB::Regex regex;
     std::string file_etctxt = CACHE::path_etcboard();
@@ -599,6 +622,10 @@ void Root::load_etc()
 
         }
     }
+
+#ifdef _DEBUG
+    std::cout << m_xml_etc << std::endl;
+#endif
 }
 
 
@@ -675,6 +702,35 @@ const std::string Root::is_board_moved( const std::string& url )
 
 
 
+// 全板の情報ファイル読み込み
+void Root::read_boardinfo_all()
+{
+    std::list< BoardBase* >::iterator it;
+    for( it = m_list_board.begin(); it != m_list_board.end(); ++it ){
+
+        ( *it )->read_info();
+    }
+}
+
+// 全ログ検索
+std::list< std::string > Root::search_cache( const std::string& query, bool mode_or, bool& stop )
+{
+    std::list< std::string > m_urllist;
+
+    std::list< BoardBase* >::iterator it;
+    for( it = m_list_board.begin(); it != m_list_board.end(); ++it ){
+
+        std::list< std::string > m_tmplist;
+        m_tmplist = ( *it )->search_cache( query, mode_or, stop );
+        m_urllist.splice( m_urllist.end(), m_tmplist );
+
+        if( stop ) break;
+    }
+
+    return m_urllist;
+}
+
+
 
 //
 // 移転テーブル保存
@@ -702,6 +758,9 @@ void Root::save_movetable()
 
     // お気に入りに登録されているURLも更新
     CORE::core_set_command( "update_favorite" );
+
+    // 履歴メニューのURLも更新
+    CORE::core_set_command( "update_history" );
 }
 
 
