@@ -1,4 +1,4 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 //
 // 画像ビュークラス
@@ -20,12 +20,17 @@ namespace IMAGE
         size_t m_length_prev;
         bool m_show_status;
         bool m_show_label;
+        bool m_show_instdialog;
 
       public:
         ImageViewMain( const std::string& url );
 
         virtual void clock_in();
         virtual void show_view();
+        virtual void scroll_up();
+        virtual void scroll_down();
+        virtual void scroll_left();
+        virtual void scroll_right();
 
       protected:
         virtual void show_status();
@@ -36,6 +41,7 @@ namespace IMAGE
         virtual bool slot_motion_notify( GdkEventMotion* event );
 
       private:
+        void show_instruct_diag();
         void set_label();
         void remove_label();
    };
