@@ -1,9 +1,8 @@
-// ライセンス: 最新のGPL
+// ライセンス: GPL2
 
 #include "viewfactory.h"
 
 #include "bbslist/bbslistview.h"
-#include "bbslist/etclistview.h"
 #include "bbslist/favoriteview.h"
 #include "bbslist/selectlistview.h"
 
@@ -26,9 +25,6 @@ SKELETON::View* CORE::ViewFactory( int type, const std::string& url, VIEWFACTORY
         case VIEW_BBSLISTVIEW:
             return new BBSLIST::BBSListViewMain( url, args.arg1, args.arg2 );
 
-        case VIEW_ETCLIST:
-            return new BBSLIST::EtcListView( url, args.arg1, args.arg2 );
-
         case VIEW_FAVORITELIST:
             return new BBSLIST::FavoriteListView( url, args.arg1, args.arg2 );
 
@@ -69,6 +65,9 @@ SKELETON::View* CORE::ViewFactory( int type, const std::string& url, VIEWFACTORY
         case VIEW_ARTICLEPREVIEW:
             return new ARTICLE::ArticleViewPreview( url );
 
+        case VIEW_ARTICLESEARCHCAHCE:
+            return new ARTICLE::ArticleViewSearchCache( url, args.arg1, ( args.arg2 == "OR" ), ( args.arg3 == "all" ) );
+
             /////////////////
 
         case VIEW_ARTICLEPOPUPHTML:
@@ -86,6 +85,9 @@ SKELETON::View* CORE::ViewFactory( int type, const std::string& url, VIEWFACTORY
         case VIEW_ARTICLEPOPUPREFER:
             return new ARTICLE::ArticleViewPopupRefer( url, args.arg1 );
 
+        case VIEW_ARTICLEPOPUPDRAWOUT:
+            return new ARTICLE::ArticleViewPopupDrawout( url, args.arg1, ( args.arg2 == "OR" ) );
+
             /////////////////
 
         case VIEW_IMAGEVIEW:
